unit LineParser;

interface

uses Math, ParserConsts;

type
  TLineParser = class
  private
    FString: WideString;
    FParsePos: integer;
    FParseEnd: integer;
    FStart: integer;
    FLength: integer;
    FPrevious: integer;
    FToken: WideChar;

    //vpeB
    function GetCanJump: boolean;
  private
    procedure ParseToken;overload;
    procedure ParseToken(const C: WideChar);overload;
    //procedure IntegerProc;
    procedure AnkProc;
    procedure HangulProc;        
    procedure DBIntProc;      
    procedure DBAlphProc;     
    procedure DBHiraProc;     
    procedure DBKanaProc;     
    procedure DBKanjiProc;    
    procedure SBKanaProc;     
    procedure UrlProc;        
    procedure UrlStartProc;   
    procedure FtpStartProc;   
    procedure HttpsStartProc; 
    procedure RtspStartProc;  
    procedure PnmStartProc;   
    procedure MmsStartProc;   
    procedure NewsStartProc;  
    procedure NntpStartProc;  
    procedure SpaceProc;      
  public
    constructor Create(const linestring: WideString; position: integer);
    //vpeB
    property Start: integer read FStart;
    property Count: integer read FLength;
    property Previous: integer read FPrevious;
    //g[NʂԂȂ̂ŁAłǂȂĂ֌WȂ
    //property Token: WideChar read FToken;
    property CanJump: boolean read GetCanJump;
  end;

implementation

{ TLineParser }

constructor TLineParser.Create(const linestring: WideString; position: integer);
begin
  FString := lineString;
  FParseEnd := Min(position, Length(FString) - 1);
  FParsePos := 0;
  FToken := #0;
  while FParsePos <= FParseEnd do begin
    FPrevious := FStart;
    FStart := FParsePos;
    ParseToken;
    FLength := FParsePos - FStart;
  end;
end;

function TLineParser.GetCanJump: boolean;
begin
  case FToken of
    toUrlStart,
    toFtpStart,
    toHttpsStart,
    toRtspStart,
    toPnmStart,
    toMmsStart,
    toNewsStart,
    toNntpStart: result := true;
    else result := false;
  end;
end;

procedure TLineParser.ParseToken;
begin
  case FString[FParsePos + 1] of
    'f':begin
      if (FParsePos + 6 < Length(FString)) and
        (FString[FParsePos + 2] = 't') and
        (FString[FParsePos + 3] = 'p') and
        (FString[FParsePos + 4] = ':') and
        (FString[FParsePos + 5] = '/') and
        (FString[FParsePos + 6] = '/') and
        IsUrlChar(FString[FParsePos + 7])
      then begin
        ParseToken(toFtpStart);
        exit;
      end;
    end;
    'h':begin
      if (FParsePos + 7 < Length(FString)) and
        (FString[FParsePos + 2] = 't') and
        (FString[FParsePos + 3] = 't') and
        (FString[FParsePos + 4] = 'p') and
        (FString[FParsePos + 5] = ':') and
        (FString[FParsePos + 6] = '/') and
        (FString[FParsePos + 7] = '/') and
        IsUrlChar(FString[FParsePos + 8])
      then begin
        ParseToken(toUrlStart);
        exit;
      end else if (FParsePos + 8 < Length(FString)) and
        (FString[FParsePos + 2] = 't') and
        (FString[FParsePos + 3] = 't') and
        (FString[FParsePos + 4] = 'p') and
        (FString[FParsePos + 5] = 's') and
        (FString[FParsePos + 6] = ':') and
        (FString[FParsePos + 7] = '/') and
        (FString[FParsePos + 8] = '/') and
        IsUrlChar(FString[FParsePos + 9])
      then begin
        ParseToken(toHttpsStart);
        exit;
      end;
    end;
    'm':begin
      if (FParsePos + 6 < Length(FString)) and
        (FString[FParsePos + 2] = 'm') and
        (FString[FParsePos + 3] = 's') and
        (FString[FParsePos + 4] = ':') and
        (FString[FParsePos + 5] = '/') and
        (FString[FParsePos + 6] = '/') and
        IsUrlChar(FString[FParsePos + 7])
      then begin
        ParseToken(toMmsStart);
        exit;
      end;
    end;
    'n':begin
      if (FParsePos + 7 < Length(FString)) and
        (FString[FParsePos + 2] = 'e') and
        (FString[FParsePos + 3] = 'w') and
        (FString[FParsePos + 4] = 's') and
        (FString[FParsePos + 5] = ':') and
        (FString[FParsePos + 6] = '/') and
        (FString[FParsePos + 7] = '/') and
        IsUrlChar(FString[FParsePos + 8])
      then begin
        ParseToken(toNewsStart);
        exit;
      end else if (FParsePos + 7 < Length(FString)) and
        (FString[FParsePos + 2] = 'n') and
        (FString[FParsePos + 3] = 't') and
        (FString[FParsePos + 4] = 'p') and
        (FString[FParsePos + 5] = ':') and
        (FString[FParsePos + 6] = '/') and
        (FString[FParsePos + 7] = '/') and
        IsUrlChar(FString[FParsePos + 8])
      then begin
        ParseToken(toNntpStart);
        exit;
      end;
    end;
    'p':begin
      if (FParsePos + 6 < Length(FString)) and
        (FString[FParsePos + 2] = 'n') and
        (FString[FParsePos + 3] = 'm') and
        (FString[FParsePos + 4] = ':') and
        (FString[FParsePos + 5] = '/') and
        (FString[FParsePos + 6] = '/') and
        IsUrlChar(FString[FParsePos + 7])
      then begin
        ParseToken(toPnmStart);
        exit;
      end;
    end;
    'r':begin
      if (FParsePos + 7 < Length(FString)) and
        (FString[FParsePos + 2] = 't') and
        (FString[FParsePos + 3] = 's') and
        (FString[FParsePos + 4] = 'p') and
        (FString[FParsePos + 5] = ':') and
        (FString[FParsePos + 6] = '/') and
        (FString[FParsePos + 7] = '/') and
        IsUrlChar(FString[FParsePos + 8])
      then begin
        ParseToken(toRtspStart);
        exit;
      end;
    end;
  end;
  ParseToken(TokenFromFirstChar(FString[FParsePos + 1]));
end;

procedure TLineParser.ParseToken(const C: WideChar);
begin
  FToken := C;
  case C of
    toSBSymbol: inc(FParsePos);
    toSBInt,//: IntegerProc;
    toAnk, toAlphabetic: AnkProc;
    toDBSymbol: inc(FParsePos);
    toDBInt: DBIntProc;
    toDBAlph: DBAlphProc;
    toDBHira: DBHiraProc;
    toDBKana: DBKanaProc;
    toDBKanji: DBKanjiProc;
    toSBKana: SBKanaProc;
    toSBKanaSymbol: inc(FParsePos);
    toUrlStart: UrlStartProc;
    toFtpStart: FtpStartProc;
    toHttpsStart: HttpsStartProc;
    toRtspStart: RtspStartProc;
    toPnmStart: PnmStartProc;
    toMmsStart: MmsStartProc;
    toNewsStart: NewsStartProc;
    toNntpStart: NntpStartProc;
    toUrl: UrlProc;
    toTab, toSpace: SpaceProc;
    toHangul: HangulProc;
    else inc(FParsePos);
  end;
end;

//procedure TLineParser.IntegerProc;
//var
//  C: WideChar;
//begin
//  repeat
//    inc(FParsePos);
//    if FParsePos >= Length(FString) then exit;
//    C := FString[FParsePos +1];
//  until (C < #$0030) or (C > #$0039);
//end;

procedure TLineParser.AnkProc;
var
  C: WideChar;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
    C := FString[FParsePos + 1];
  until (not (TokenFromFirstChar(C) in [toSBInt,toAnk,toAlphabetic])) ;
end;

procedure TLineParser.DBIntProc;
var
  C: WideChar;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
    C := FString[FParsePos + 1];
  until (C < #$FF10) or (C > #$FF19);
end;

procedure TLineParser.DBAlphProc;
var
  C: WideChar;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
    C := FString[FParsePos + 1];
  until (C < #$FF21) or ((C < #$FF41) and (C > #$FF3A)) or (C > #$FF5A);
end;

procedure TLineParser.DBHiraProc;
var
  C: WideChar;
  b: boolean;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
    C := FString[FParsePos + 1];
    case TokenFromFirstChar(C) of
     toDBHira: b := true;
     else b := (C = #$30FC);
    end;
    {case GetUnicodeSubset(C) of
      usHiragana: b := true;
      else begin
        b := (C = #$30FC);
      end;
    end;  }
  until not b;
end;

procedure TLineParser.DBKanaProc;
var
  C: WideChar;
  b: boolean;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
    C := FString[FParsePos + 1];
    case TokenFromFirstChar(C) of
      toDBKana: b := true;
      else begin
        b := (#$3099 <= C) and (C <= #$309C);
      end;
    end;
    {case GetUnicodeSubset(C) of
      usKatakana: b := true;
      else begin
        b := (#$3099 <= C) and (C <= #$309C);
      end;
    end;}
  until not b;
  {case GetUnicodeSubset(C) of
    usHiragana: DBHiraProc;
    usCJKIdeographs, usCJKCompatibilityIdeographs: DBKanjiProc;
    usCJKSymbolsPunctuation: begin
      case C of
        #$3003, #$3005..#$3007: DBKanjiProc;//'V''X''Y''Z'
      end;
    end;
    usHalfwidthFullwidthForms: begin
      case C of
        #$FF66..#$FF9F: SBKanaProc;
      end;
    end;
  end; }
end;

procedure TLineParser.DBKanjiProc;
var
  C: WideChar;
  b: boolean;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
    C := FString[FParsePos + 1];
    case TokenFromFirstChar(C) of
      toDBKanji: b := true;
      else b := false;
    end;
{    case GetUnicodeSubset(C) of
      usCJKIdeographs,
      usCJKCompatibilityIdeographs: b := true;
      usCJKSymbolsPunctuation: begin
        case C of
          #$3003, #$3005..#$3007: b := true;//'V''X''Y''Z'
          else b := false
        end;
      end;
      else b := false;
    end;}
  until not b;
  {case GetUnicodeSubset(C) of
    usHiragana: DBHiraProc;
    usKatakana: DBKanaProc;
    usHalfwidthFullwidthForms: begin
      case C of
        #$FF66..#$FF9F: SBKanaProc;
      end;
    end;
  end;}
end;

procedure TLineParser.SBKanaProc;
var
  C: WideChar;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
    C := FString[FParsePos + 1];

  until (C < #$FF66) or (C > #$FF9F);
  {case GetUnicodeSubset(C) of
    usHiragana: DBHiraProc;
    usKatakana: DBKanaProc;
    usCJKIdeographs, usCJKCompatibilityIdeographs: DBKanjiProc;
    usCJKSymbolsPunctuation: begin
      case C of
        #$3003, #$3005..#$3007: DBKanjiProc;//'V''X''Y''Z'
      end;
    end;
  end;  }
end;

procedure TLineParser.UrlProc;
var
  C: WideChar;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
    C := FString[FParsePos + 1];
  until not IsUrlChar(C);
end;

procedure TLineParser.UrlStartProc;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
  until (FParsePos >= 7) and
    (FString[FParsePos - 6] = 'h') and (FString[FParsePos - 5] = 't') and
    (FString[FParsePos - 4] = 't') and (FString[FParsePos - 3] = 'p') and
    (FString[FParsePos - 2] = ':') and (FString[FParsePos - 1] = '/') and
    (FString[FParsePos] = '/');
  UrlProc;
end;

procedure TLineParser.FtpStartProc;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
  until (FParsePos >= 6) and
    (FString[FParsePos - 5] = 'f') and (FString[FParsePos - 4] = 't') and
    (FString[FParsePos - 3] = 'p') and (FString[FParsePos - 2] = ':') and
    (FString[FParsePos - 1] = '/') and (FString[FParsePos] = '/');
  UrlProc;
end;

procedure TLineParser.HttpsStartProc;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
  until (FParsePos >= 8) and
    (FString[FParsePos - 7] = 'h') and (FString[FParsePos - 6] = 't') and
    (FString[FParsePos - 5] = 't') and (FString[FParsePos - 4] = 'p') and
    (FString[FParsePos - 3] = 's') and (FString[FParsePos - 2] = ':') and
    (FString[FParsePos - 1] = '/') and (FString[FParsePos] = '/');
  UrlProc;
end;

procedure TLineParser.RtspStartProc;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
  until (FParsePos >= 7) and
    (FString[FParsePos - 6] = 'r') and (FString[FParsePos - 5] = 't') and
    (FString[FParsePos - 4] = 's') and (FString[FParsePos - 3] = 'p') and
    (FString[FParsePos - 2] = ':') and (FString[FParsePos - 1] = '/') and
    (FString[FParsePos] = '/');
  UrlProc;
end;

procedure TLineParser.PnmStartProc;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
  until (FParsePos >= 6) and
    (FString[FParsePos - 5] = 'p') and (FString[FParsePos - 4] = 'n') and
    (FString[FParsePos - 3] = 'm') and (FString[FParsePos - 2] = ':') and
    (FString[FParsePos - 1] = '/') and (FString[FParsePos] = '/');
  UrlProc;
end;

procedure TLineParser.MmsStartProc;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
  until (FParsePos >= 6) and
    (FString[FParsePos - 5] = 'm') and (FString[FParsePos - 4] = 'm') and
    (FString[FParsePos - 3] = 's') and (FString[FParsePos - 2] = ':') and
    (FString[FParsePos - 1] = '/') and (FString[FParsePos] = '/');
  UrlProc;
end;

procedure TLineParser.NewsStartProc;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
  until (FParsePos >= 7) and
    (FString[FParsePos - 6] = 'n') and (FString[FParsePos - 5] = 'e') and
    (FString[FParsePos - 4] = 'w') and (FString[FParsePos - 3] = 's') and
    (FString[FParsePos - 2] = ':') and (FString[FParsePos - 1] = '/') and
    (FString[FParsePos] = '/');
  UrlProc;
end;

procedure TLineParser.NntpStartProc;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
  until (FParsePos >= 7) and
    (FString[FParsePos - 6] = 'n') and (FString[FParsePos - 5] = 'n') and
    (FString[FParsePos - 4] = 't') and (FString[FParsePos - 3] = 'p') and
    (FString[FParsePos - 2] = ':') and (FString[FParsePos - 1] = '/') and
    (FString[FParsePos] = '/');
  UrlProc;
end;

procedure TLineParser.SpaceProc;
var
  C: WideChar;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
    C := FString[FParsePos + 1];
  until (C <> #$0009) and (C <> #$0020);
end;

procedure TLineParser.HangulProc;
var
  C: WideChar;
begin
  repeat
    inc(FParsePos);
    if FParsePos >= Length(FString) then exit;
    C := FString[FParsePos + 1];
  until (C < #$AC00) or (C > #$D7A3);
end;

end.
