unit ModeHtml;

interface

uses NotePadMode, NotePadString;

type
  TModeHtml = class(TNotePadMode)
  protected
    procedure InitReserveList;override;
  public
    function NewParser(target: TNotePadString): TNotePadParser;override;
  end;

  TParserHtml = class(TNotePadParser)
  private
  protected
    procedure ParseToken(AHttp: boolean);overload;override;
    procedure ParseToken(const C: WideChar);overload;override;
  public
    constructor Create(target: TNotePadString; mode: TModeHtml);
  end;

implementation

uses NotePadClasses, ParserConsts;

//http://php.planetmirror.com/manual/ja/reserved.php
const
  ss1: array[0..66] of WideString =(
    'abstract',
    'and',
    'array',
    'as',
    'break',
    'case',
    'catch',
    'cfunction',
    'class',
    'clone',
    'const',
    'continue',
    'declare',
    'default',
    'die',
    'do',
    'echo',
    'else',
    'elseif',
    'empty',
    'enddeclare',
    'endfor',
    'endforeach',
    'endif',
    'endswitch',
    'endwhile',
    'eval',
    'exception',
    'exit',
    'extends',
    'final',
    'for',
    'foreach',
    'function',
    'global',
    'if',
    'implements',
    'include',
    'include_once',
    'interface',
    'isset',
    'list',
    'new',
    'old_function',
    'or',
    'php_user_filter',
    'print',
    'private',
    'protected',
    'public',
    'require',
    'require_once',
    'return',
    'static',
    'switch',
    'throw',
    'try',
    'unset',
    'use',
    'var',
    'while',
    'xor',
    '__CLASS__',
    '__FILE__',
    '__FUNCTION__',
    '__LINE__',
    '__METHOD__');

  ss2: array[0..1077] of WideString =(
'abs',
'acos',
'addcslashes',
'addslashes',
'aggregate',
'aggregate_methods',
'aggregate_methods_by_list',
'aggregate_methods_by_regexp',
'aggregate_properties',
'aggregate_properties_by_list',
'aggregate_properties_by_regexp',
'aggregation_info',
'array_change_key_case',
'array_chunk',
'array_count_values',
'array_diff',
'array_diff_assoc',
'array_fill',
'array_filter',
'array_flip',
'array_intersect',
'array_intersect_assoc',
'array_key_exists',
'array_keys',
'array_map',
'array_merge',
'array_merge_recursive',
'array_multisort',
'array_pad',
'array_pop',
'array_push',
'array_rand',
'array_reduce',
'array_reverse',
'array_search',
'array_shift',
'array_slice',
'array_splice',
'array_sum',
'array_unique',
'array_unshift',
'array_values',
'array_walk',
'arsort',
'asin',
'asort',
'assert',
'assert_options',
'atan',
'atan2',
'base64_decode',
'base64_encode',
'base_convert',
'basename',
'bcadd',
'bccomp',
'bcdiv',
'bcmod',
'bcmul',
'bcpow',
'bcscale',
'bcsqrt',
'bcsub',
'bin2hex',
'bind_textdomain_codeset',
'bindec',
'bindtextdomain',
'call_user_func',
'call_user_func_array',
'call_user_method',
'call_user_method_array',
'ceil',
'chdir',
'checkdate',
'chgrp',
'chmod',
'chop',
'chown',
'chr',
'chunk_split',
'clearstatcache',
'closedir',
'closelog',
'compact',
'connection_aborted',
'connection_status',
'constant',
'convert_cyr_string',
'copy',
'cos',
'cosh',
'count',
'count_chars',
'crc32',
'crypt',
'ctype_alnum',
'ctype_alpha',
'ctype_cntrl',
'ctype_digit',
'ctype_graph',
'ctype_lower',
'ctype_print',
'ctype_punct',
'ctype_space',
'ctype_upper',
'ctype_xdigit',
'current',
'date',
'dcgettext',
'deaggregate',
'debug_zval_dump',
'decbin',
'dechex',
'decoct',
'define_syslog_variables',
'deg2rad',
'dgettext',
'dir',
'dirname',
'disk_free_space',
'disk_total_space',
'diskfreespace',
'dl',
'dngettext',
'doubleval',
'end',
'ereg',
'ereg_replace',
'eregi',
'eregi_replace',
'error_log',
'escapeshellarg',
'escapeshellcmd',
'exec',
'exif_imagetype',
'exif_read_data',
'exif_tagname',
'exif_thumbnail',
'exp',
'explode',
'extract',
'ezmlm_hash',
'fclose',
'feof',
'fflush',
'fgetc',
'fgetcsv',
'fgets',
'fgetss',
'file',
'file_exists',
'file_get_contents',
'fileatime',
'filectime',
'filegroup',
'fileinode',
'filemtime',
'fileowner',
'fileperms',
'filesize',
'filetype',
'floatval',
'flock',
'floor',
'flush',
'fmod',
'fopen',
'fpassthru',
'fputs',
'fread',
'fscanf',
'fseek',
'fsockopen',
'fstat',
'ftell',
'ftp_cdup',
'ftp_chdir',
'ftp_close',
'ftp_connect',
'ftp_delete',
'ftp_exec',
'ftp_fget',
'ftp_fput',
'ftp_get',
'ftp_get_option',
'ftp_login',
'ftp_mdtm',
'ftp_mkdir',
'ftp_nb_continue',
'ftp_nb_fget',
'ftp_nb_fput',
'ftp_nb_get',
'ftp_nb_put',
'ftp_nlist',
'ftp_pasv',
'ftp_put',
'ftp_pwd',
'ftp_quit',
'ftp_rawlist',
'ftp_rename',
'ftp_rmdir',
'ftp_set_option',
'ftp_site',
'ftp_size',
'ftp_ssl_connect',
'ftp_systype',
'ftruncate',
'fwrite',
'gd_info',
'get_browser',
'get_cfg_var',
'get_current_user',
'get_html_translation_table',
'get_include_path',
'get_magic_quotes_gpc',
'get_magic_quotes_runtime',
'get_meta_tags',
'getcwd',
'getdate',
'getenv',
'gethostbyaddr',
'gethostbyname',
'gethostbynamel',
'getimagesize',
'getlastmod',
'getmygid',
'getmyinode',
'getmypid',
'getmyuid',
'getprotobyname',
'getprotobynumber',
'getrandmax',
'getservbyname',
'getservbyport',
'gettext',
'gettimeofday',
'gettype',
'glob',
'gmdate',
'gmmktime',
'gmstrftime',
'gzclose',
'gzcompress',
'gzdeflate',
'gzencode',
'gzeof',
'gzfile',
'gzgetc',
'gzgets',
'gzgetss',
'gzinflate',
'gzopen',
'gzpassthru',
'gzputs',
'gzread',
'gzrewind',
'gzseek',
'gztell',
'gzuncompress',
'gzwrite',
'header',
'headers_sent',
'hebrev',
'hebrevc',
'hexdec',
'highlight_file',
'highlight_string',
'html_entity_decode',
'htmlentities',
'htmlspecialchars',
'i18n_convert',
'i18n_discover_encoding',
'i18n_http_input',
'i18n_http_output',
'i18n_internal_encoding',
'i18n_ja_jp_hantozen',
'i18n_mime_header_decode',
'i18n_mime_header_encode',
'ignore_user_abort',
'image2wbmp',
'image_type_to_mime_type',
'imagealphablending',
'imageantialias',
'imagearc',
'imagechar',
'imagecharup',
'imagecolorallocate',
'imagecolorallocatealpha',
'imagecolorat',
'imagecolorclosest',
'imagecolorclosestalpha',
'imagecolordeallocate',
'imagecolorexact',
'imagecolorexactalpha',
'imagecolormatch',
'imagecolorresolve',
'imagecolorresolvealpha',
'imagecolorset',
'imagecolorsforindex',
'imagecolorstotal',
'imagecolortransparent',
'imagecopy',
'imagecopymerge',
'imagecopymergegray',
'imagecopyresampled',
'imagecopyresized',
'imagecreate',
'imagecreatefromgd',
'imagecreatefromgd2',
'imagecreatefromgd2part',
'imagecreatefromgif',
'imagecreatefromjpeg',
'imagecreatefrompng',
'imagecreatefromstring',
'imagecreatefromwbmp',
'imagecreatefromxbm',
'imagecreatetruecolor',
'imagedashedline',
'imagedestroy',
'imageellipse',
'imagefill',
'imagefilledarc',
'imagefilledellipse',
'imagefilledpolygon',
'imagefilledrectangle',
'imagefilltoborder',
'imagefontheight',
'imagefontwidth',
'imageftbbox',
'imagefttext',
'imagegammacorrect',
'imagegd',
'imagegd2',
'imagegif',
'imageinterlace',
'imageistruecolor',
'imagejpeg',
'imagelayereffect',
'imageline',
'imageloadfont',
'imagepalettecopy',
'imagepng',
'imagepolygon',
'imagepsbbox',
'imagepsencodefont',
'imagepsextendfont',
'imagepsfreefont',
'imagepsloadfont',
'imagepsslantfont',
'imagepstext',
'imagerectangle',
'imagerotate',
'imagesavealpha',
'imagesetbrush',
'imagesetpixel',
'imagesetstyle',
'imagesetthickness',
'imagesettile',
'imagestring',
'imagestringup',
'imagesx',
'imagesy',
'imagetruecolortopalette',
'imagettfbbox',
'imagettftext',
'imagetypes',
'imagewbmp',
'imap_8bit',
'imap_alerts',
'imap_append',
'imap_base64',
'imap_binary',
'imap_body',
'imap_bodystruct',
'imap_check',
'imap_clearflag_full',
'imap_close',
'imap_create',
'imap_createmailbox',
'imap_delete',
'imap_deletemailbox',
'imap_errors',
'imap_expunge',
'imap_fetch_overview',
'imap_fetchbody',
'imap_fetchheader',
'imap_fetchstructure',
'imap_fetchtext',
'imap_get_quota',
'imap_get_quotaroot',
'imap_getmailboxes',
'imap_getsubscribed',
'imap_header',
'imap_headerinfo',
'imap_headers',
'imap_last_error',
'imap_list',
'imap_listmailbox',
'imap_listsubscribed',
'imap_lsub',
'imap_mail',
'imap_mail_compose',
'imap_mail_copy',
'imap_mail_move',
'imap_mailboxmsginfo',
'imap_mime_header_decode',
'imap_msgno',
'imap_num_msg',
'imap_num_recent',
'imap_open',
'imap_ping',
'imap_qprint',
'imap_rename',
'imap_renamemailbox',
'imap_reopen',
'imap_rfc822_parse_adrlist',
'imap_rfc822_parse_headers',
'imap_rfc822_write_address',
'imap_scan',
'imap_scanmailbox',
'imap_search',
'imap_set_quota',
'imap_setacl',
'imap_setflag_full',
'imap_sort',
'imap_status',
'imap_subscribe',
'imap_thread',
'imap_timeout',
'imap_uid',
'imap_undelete',
'imap_unsubscribe',
'imap_utf7_decode',
'imap_utf7_encode',
'imap_utf8',
'implode',
'import_request_variables',
'in_array',
'ini_alter',
'ini_get',
'ini_get_all',
'ini_restore',
'ini_set',
'intval',
'ip2long',
'iptcembed',
'iptcparse',
'is_array',
'is_bool',
'is_callable',
'is_dir',
'is_double',
'is_file',
'is_finite',
'is_float',
'is_infinite',
'is_int',
'is_integer',
'is_link',
'is_long',
'is_nan',
'is_null',
'is_numeric',
'is_object',
'is_readable',
'is_real',
'is_resource',
'is_scalar',
'is_string',
'is_uploaded_file',
'is_writable',
'is_writeable',
'join',
'jpeg2wbmp',
'key',
'key_exists',
'krsort',
'ksort',
'lcg_value',
'levenshtein',
'localeconv',
'localtime',
'log',
'log10',
'long2ip',
'lstat',
'ltrim',
'magic_quotes_runtime',
'mail',
'max',
'mb_convert_case',
'mb_convert_encoding',
'mb_convert_kana',
'mb_convert_variables',
'mb_decode_mimeheader',
'mb_decode_numericentity',
'mb_detect_encoding',
'mb_detect_order',
'mb_encode_mimeheader',
'mb_encode_numericentity',
'mb_ereg',
'mb_ereg_match',
'mb_ereg_replace',
'mb_ereg_search',
'mb_ereg_search_getpos',
'mb_ereg_search_getregs',
'mb_ereg_search_init',
'mb_ereg_search_pos',
'mb_ereg_search_regs',
'mb_ereg_search_setpos',
'mb_eregi',
'mb_eregi_replace',
'mb_get_info',
'mb_http_input',
'mb_http_output',
'mb_internal_encoding',
'mb_language',
'mb_output_handler',
'mb_parse_str',
'mb_preferred_mime_name',
'mb_regex_encoding',
'mb_regex_set_options',
'mb_send_mail',
'mb_split',
'mb_strcut',
'mb_strimwidth',
'mb_strlen',
'mb_strpos',
'mb_strrpos',
'mb_strtolower',
'mb_strtoupper',
'mb_strwidth',
'mb_substitute_character',
'mb_substr',
'mb_substr_count',
'mbereg',
'mbereg_match',
'mbereg_replace',
'mbereg_search',
'mbereg_search_getpos',
'mbereg_search_getregs',
'mbereg_search_init',
'mbereg_search_pos',
'mbereg_search_regs',
'mbereg_search_setpos',
'mberegi',
'mberegi_replace',
'mbregex_encoding',
'mbsplit',
'mbstrcut',
'mbstrlen',
'mbstrpos',
'mbstrrpos',
'mbsubstr',
'md5',
'md5_file',
'metaphone',
'microtime',
'min',
'mkdir',
'mktime',
'move_uploaded_file',
'mt_getrandmax',
'mt_rand',
'mt_srand',
'mysql',
'mysql_affected_rows',
'mysql_client_encoding',
'mysql_close',
'mysql_connect',
'mysql_data_seek',
'mysql_db_name',
'mysql_db_query',
'mysql_dbname',
'mysql_errno',
'mysql_error',
'mysql_escape_string',
'mysql_fetch_array',
'mysql_fetch_assoc',
'mysql_fetch_field',
'mysql_fetch_lengths',
'mysql_fetch_object',
'mysql_fetch_row',
'mysql_field_flags',
'mysql_field_len',
'mysql_field_name',
'mysql_field_seek',
'mysql_field_table',
'mysql_field_type',
'mysql_fieldflags',
'mysql_fieldlen',
'mysql_fieldname',
'mysql_fieldtable',
'mysql_fieldtype',
'mysql_free_result',
'mysql_freeresult',
'mysql_get_client_info',
'mysql_get_host_info',
'mysql_get_proto_info',
'mysql_get_server_info',
'mysql_info',
'mysql_insert_id',
'mysql_list_dbs',
'mysql_list_fields',
'mysql_list_processes',
'mysql_list_tables',
'mysql_listdbs',
'mysql_listfields',
'mysql_listtables',
'mysql_num_fields',
'mysql_num_rows',
'mysql_numfields',
'mysql_numrows',
'mysql_pconnect',
'mysql_ping',
'mysql_query',
'mysql_real_escape_string',
'mysql_result',
'mysql_select_db',
'mysql_selectdb',
'mysql_stat',
'mysql_table_name',
'mysql_tablename',
'mysql_thread_id',
'mysql_unbuffered_query',
'natcasesort',
'natsort',
'next',
'ngettext',
'nl2br',
'number_format',
'ob_clean',
'ob_end_clean',
'ob_end_flush',
'ob_flush',
'ob_get_clean',
'ob_get_contents',
'ob_get_flush',
'ob_get_length',
'ob_get_level',
'ob_get_status',
'ob_gzhandler',
'ob_implicit_flush',
'ob_list_handlers',
'ob_start',
'octdec',
'opendir',
'openlog',
'openssl_csr_export',
'openssl_csr_export_to_file',
'openssl_csr_new',
'openssl_csr_sign',
'openssl_error_string',
'openssl_free_key',
'openssl_get_privatekey',
'openssl_get_publickey',
'openssl_open',
'openssl_pkcs7_decrypt',
'openssl_pkcs7_encrypt',
'openssl_pkcs7_sign',
'openssl_pkcs7_verify',
'openssl_pkey_export',
'openssl_pkey_export_to_file',
'openssl_pkey_free',
'openssl_pkey_get_private',
'openssl_pkey_get_public',
'openssl_pkey_new',
'openssl_private_decrypt',
'openssl_private_encrypt',
'openssl_public_decrypt',
'openssl_public_encrypt',
'openssl_seal',
'openssl_sign',
'openssl_verify',
'openssl_x509_check_private_key',
'openssl_x509_checkpurpose',
'openssl_x509_export',
'openssl_x509_export_to_file',
'openssl_x509_free',
'openssl_x509_parse',
'openssl_x509_read',
'ord',
'output_add_rewrite_var',
'output_reset_rewrite_vars',
'overload',
'pack',
'parse_ini_file',
'parse_str',
'parse_url',
'passthru',
'pathinfo',
'pclose',
'pdf_activate_item',
'pdf_add_bookmark',
'pdf_add_launchlink',
'pdf_add_locallink',
'pdf_add_nameddest',
'pdf_add_note',
'pdf_add_pdflink',
'pdf_add_thumbnail',
'pdf_add_weblink',
'pdf_arc',
'pdf_arcn',
'pdf_attach_file',
'pdf_begin_document',
'pdf_begin_font',
'pdf_begin_glyph',
'pdf_begin_item',
'pdf_begin_layer',
'pdf_begin_page',
'pdf_begin_page_ext',
'pdf_begin_pattern',
'pdf_begin_template',
'pdf_circle',
'pdf_clip',
'pdf_close',
'pdf_close_image',
'pdf_close_pdi',
'pdf_close_pdi_page',
'pdf_closepath',
'pdf_closepath_fill_stroke',
'pdf_closepath_stroke',
'pdf_concat',
'pdf_continue_text',
'pdf_create_action',
'pdf_create_annotation',
'pdf_create_bookmark',
'pdf_create_field',
'pdf_create_fieldgroup',
'pdf_create_gstate',
'pdf_create_pvf',
'pdf_create_textflow',
'pdf_curveto',
'pdf_define_layer',
'pdf_delete',
'pdf_delete_pvf',
'pdf_delete_textflow',
'pdf_encoding_set_char',
'pdf_end_document',
'pdf_end_font',
'pdf_end_glyph',
'pdf_end_item',
'pdf_end_layer',
'pdf_end_page',
'pdf_end_page_ext',
'pdf_end_pattern',
'pdf_end_template',
'pdf_endpath',
'pdf_fill',
'pdf_fill_imageblock',
'pdf_fill_pdfblock',
'pdf_fill_stroke',
'pdf_fill_textblock',
'pdf_findfont',
'pdf_fit_image',
'pdf_fit_pdi_page',
'pdf_fit_textflow',
'pdf_fit_textline',
'pdf_get_apiname',
'pdf_get_buffer',
'pdf_get_errmsg',
'pdf_get_errnum',
'pdf_get_parameter',
'pdf_get_pdi_parameter',
'pdf_get_pdi_value',
'pdf_get_value',
'pdf_info_textflow',
'pdf_initgraphics',
'pdf_lineto',
'pdf_load_font',
'pdf_load_iccprofile',
'pdf_load_image',
'pdf_makespotcolor',
'pdf_moveto',
'pdf_new',
'pdf_open_ccitt',
'pdf_open_file',
'pdf_open_image',
'pdf_open_image_file',
'pdf_open_pdi',
'pdf_open_pdi_page',
'pdf_place_image',
'pdf_place_pdi_page',
'pdf_process_pdi',
'pdf_rect',
'pdf_restore',
'pdf_resume_page',
'pdf_rotate',
'pdf_save',
'pdf_scale',
'pdf_set_border_color',
'pdf_set_border_dash',
'pdf_set_border_style',
'pdf_set_gstate',
'pdf_set_info',
'pdf_set_layer_dependency',
'pdf_set_parameter',
'pdf_set_text_pos',
'pdf_set_value',
'pdf_setcolor',
'pdf_setdash',
'pdf_setdashpattern',
'pdf_setflat',
'pdf_setfont',
'pdf_setlinecap',
'pdf_setlinejoin',
'pdf_setlinewidth',
'pdf_setmatrix',
'pdf_setmiterlimit',
'pdf_setpolydash',
'pdf_shading',
'pdf_shading_pattern',
'pdf_shfill',
'pdf_show',
'pdf_show_boxed',
'pdf_show_xy',
'pdf_skew',
'pdf_stringwidth',
'pdf_stroke',
'pdf_suspend_page',
'pdf_translate',
'pdf_utf16_to_utf8',
'pdf_utf8_to_utf16',
'pfsockopen',
'php_ini_scanned_files',
'php_logo_guid',
'php_sapi_name',
'php_uname',
'phpcredits',
'phpinfo',
'phpversion',
'pi',
'png2wbmp',
'popen',
'pos',
'pow',
'preg_grep',
'preg_match',
'preg_match_all',
'preg_quote',
'preg_replace',
'preg_replace_callback',
'preg_split',
'prev',
'print_r',
'printf',
'proc_close',
'proc_open',
'putenv',
'quoted_printable_decode',
'quotemeta',
'rad2deg',
'rand',
'range',
'rawurldecode',
'rawurlencode',
'read_exif_data',
'readdir',
'readfile',
'readgzfile',
'realpath',
'register_shutdown_function',
'register_tick_function',
'rename',
'reset',
'restore_include_path',
'rewind',
'rewinddir',
'rmdir',
'round',
'rsort',
'rtrim',
'serialize',
'session_cache_expire',
'session_cache_limiter',
'session_commit',
'session_decode',
'session_destroy',
'session_encode',
'session_get_cookie_params',
'session_id',
'session_is_registered',
'session_module_name',
'session_name',
'session_regenerate_id',
'session_register',
'session_save_path',
'session_set_cookie_params',
'session_set_save_handler',
'session_start',
'session_unregister',
'session_unset',
'session_write_close',
'set_file_buffer',
'set_include_path',
'set_magic_quotes_runtime',
'set_socket_blocking',
'set_time_limit',
'setcookie',
'setlocale',
'settype',
'sha1',
'sha1_file',
'shell_exec',
'show_source',
'shuffle',
'similar_text',
'sin',
'sinh',
'sizeof',
'sleep',
'socket_accept',
'socket_bind',
'socket_clear_error',
'socket_close',
'socket_connect',
'socket_create',
'socket_create_listen',
'socket_create_pair',
'socket_get_option',
'socket_get_status',
'socket_getopt',
'socket_getpeername',
'socket_getsockname',
'socket_iovec_add',
'socket_iovec_alloc',
'socket_iovec_delete',
'socket_iovec_fetch',
'socket_iovec_free',
'socket_iovec_set',
'socket_last_error',
'socket_listen',
'socket_read',
'socket_readv',
'socket_recv',
'socket_recvfrom',
'socket_select',
'socket_send',
'socket_sendmsg',
'socket_sendto',
'socket_set_block',
'socket_set_blocking',
'socket_set_nonblock',
'socket_set_option',
'socket_set_timeout',
'socket_setopt',
'socket_shutdown',
'socket_strerror',
'socket_write',
'socket_writev',
'sort',
'soundex',
'split',
'spliti',
'sprintf',
'sql_regcase',
'sqrt',
'srand',
'sscanf',
'stat',
'str_pad',
'str_repeat',
'str_replace',
'str_rot13',
'str_shuffle',
'str_word_count',
'strchr',
'strcoll',
'strcspn',
'stream_context_create',
'stream_context_get_options',
'stream_context_set_option',
'stream_context_set_params',
'stream_filter_append',
'stream_filter_prepend',
'stream_get_meta_data',
'stream_register_wrapper',
'stream_select',
'stream_set_blocking',
'stream_set_timeout',
'stream_set_write_buffer',
'stream_wrapper_register',
'strftime',
'strip_tags',
'stripcslashes',
'stripslashes',
'stristr',
'strnatcasecmp',
'strnatcmp',
'strpos',
'strrchr',
'strrev',
'strrpos',
'strspn',
'strstr',
'strtok',
'strtolower',
'strtotime',
'strtoupper',
'strtr',
'strval',
'substr',
'substr_count',
'substr_replace',
'syslog',
'system',
'tan',
'tanh',
'tempnam',
'textdomain',
'time',
'tmpfile',
'token_get_all',
'token_name',
'touch',
'trim',
'uasort',
'ucfirst',
'ucwords',
'uksort',
'umask',
'uniqid',
'unlink',
'unpack',
'unregister_tick_function',
'unserialize',
'urldecode',
'urlencode',
'usleep',
'usort',
'utf8_decode',
'utf8_encode',
'var_dump',
'var_export',
'version_compare',
'vprintf',
'vsprintf',
'wordwrap',
'xml_error_string',
'xml_get_current_byte_index',
'xml_get_current_column_number',
'xml_get_current_line_number',
'xml_get_error_code',
'xml_parse',
'xml_parse_into_struct',
'xml_parser_create',
'xml_parser_create_ns',
'xml_parser_free',
'xml_parser_get_option',
'xml_parser_set_option',
'xml_set_character_data_handler',
'xml_set_default_handler',
'xml_set_element_handler',
'xml_set_end_namespace_decl_handler',
'xml_set_external_entity_ref_handler',
'xml_set_notation_decl_handler',
'xml_set_object',
'xml_set_processing_instruction_handler',
'xml_set_start_namespace_decl_handler',
'xml_set_unparsed_entity_decl_handler',
'xslt_backend_info',
'xslt_backend_name',
'xslt_backend_version',
'xslt_create',
'xslt_errno',
'xslt_error',
'xslt_free',
'xslt_process',
'xslt_set_base',
'xslt_set_encoding',
'xslt_set_error_handler',
'xslt_set_log',
'xslt_set_object',
'xslt_set_sax_handlers',
'xslt_set_scheme_handlers',
'xslt_setopt',
'zend_logo_guid',
'zlib_get_coding_type');


  ss3: array[0..235] of WideString =(
'$GLOBALS',
'$_COOKIE',
'$_ENV',
'$_FILES',
'$_GET',
'$_POST',
'$_REQUEST',
'$_SERVER',
'$_SESSION',
'DEFAULT_INCLUDE_PATH',
'E_ALL',
'E_COMPILE_ERROR',
'E_COMPILE_WARNING',
'E_CORE_ERROR',
'E_CORE_WARNING',
'E_ERROR',
'E_NOTICE',
'E_PARSE',
'E_STRICT',
'E_USER_ERROR',
'E_USER_NOTICE',
'E_USER_WARNING',
'E_WARNING',
'PEAR_EXTENSION_DIR',
'PEAR_INSTALL_DIR',
'PHP_BINDIR',
'PHP_CONFIG_FILE_PATH',
'PHP_DATADIR',
'PHP_EXTENSION_DIR',
'PHP_INT_MAX',
'PHP_INT_SIZE',
'PHP_LIBDIR',
'PHP_LOCALSTATEDIR',
'PHP_OS',
'PHP_OUTPUT_HANDLER_CONT',
'PHP_OUTPUT_HANDLER_END',
'PHP_OUTPUT_HANDLER_START',
'PHP_SYSCONFDIR',
'PHP_VERSION',
'ABDAY_1',
'ABDAY_2',
'ABDAY_3',
'ABDAY_4',
'ABDAY_5',
'ABDAY_6',
'ABDAY_7',
'ABMON_1',
'ABMON_10',
'ABMON_11',
'ABMON_12',
'ABMON_2',
'ABMON_3',
'ABMON_4',
'ABMON_5',
'ABMON_6',
'ABMON_7',
'ABMON_8',
'ABMON_9',
'ALT_DIGITS',
'AM_STR',
'ASSERT_ACTIVE',
'ASSERT_BAIL',
'ASSERT_CALLBACK',
'ASSERT_QUIET_EVAL',
'ASSERT_WARNING',
'CASE_LOWER',
'CASE_UPPER',
'CHAR_MAX',
'CODESET',
'CONNECTION_NORMAL',
'CONNECTION_TIMEOUT',
'COUNT_NORMAL',
'COUNT_RECURSIVE',
'CREDITS_ALL',
'CREDITS_DOCS',
'CREDITS_FULLPAGE',
'CREDITS_GENERAL',
'CREDITS_GROUP',
'CREDITS_MODULES',
'CREDITS_QA',
'CREDITS_SAPI',
'CRNCYSTR',
'CRYPT_BLOWFISH',
'CRYPT_EXT_DES',
'CRYPT_MD5',
'CRYPT_SALT_LENGTH',
'CRYPT_STD_DES',
'CURRENCY_SYMBOL',
'D_FMT',
'D_T_FMT',
'DAY_1',
'DAY_2',
'DAY_3',
'DAY_4',
'DAY_5',
'DAY_6',
'DAY_7',
'DECIMAL_POINT',
'DIRECTORY_SEPARATOR',
'ENT_COMPAT',
'ENT_NOQUOTES',
'ENT_QUOTES',
'ERA',
'ERA_D_FMT',
'ERA_D_T_FMT',
'ERA_T_FMT',
'ERA_YEAR',
'EXTR_IF_EXISTS',
'EXTR_OVERWRITE',
'EXTR_PREFIX_ALL',
'EXTR_PREFIX_IF_EXISTS',
'EXTR_PREFIX_INVALID',
'EXTR_PREFIX_SAME',
'EXTR_SKIP',
'FRAC_DIGITS',
'GROUPING',
'HTML_ENTITIES',
'HTML_SPECIALCHARS',
'INFO_ALL',
'INFO_CONFIGURATION',
'INFO_CREDITS',
'INFO_ENVIRONMENT',
'INFO_GENERAL',
'INFO_LICENSE',
'INFO_MODULES',
'INFO_VARIABLES',
'INI_ALL',
'INI_PERDIR',
'INI_SYSTEM',
'INI_USER',
'INT_CURR_SYMBOL',
'INT_FRAC_DIGITS',
'LC_ALL',
'LC_COLLATE',
'LC_CTYPE',
'LC_MESSAGES',
'LC_MONETARY',
'LC_NUMERIC',
'LC_TIME',
'LOCK_EX',
'LOCK_NB',
'LOCK_SH',
'LOCK_UN',
'LOG_ALERT',
'LOG_AUTH',
'LOG_AUTHPRIV',
'LOG_CONS',
'LOG_CRIT',
'LOG_CRON',
'LOG_DAEMON',
'LOG_DEBUG',
'LOG_EMERG',
'LOG_ERR',
'LOG_INFO',
'LOG_KERN',
'LOG_LOCAL0',
'LOG_LOCAL1',
'LOG_LOCAL2',
'LOG_LOCAL3',
'LOG_LOCAL4',
'LOG_LOCAL5',
'LOG_LOCAL6',
'LOG_LOCAL7',
'LOG_LPR',
'LOG_MAIL',
'LOG_NDELAY',
'LOG_NEWS',
'LOG_NOTICE',
'LOG_NOWAIT',
'LOG_ODELAY',
'LOG_PERROR',
'LOG_PID',
'LOG_SYSLOG',
'LOG_USER',
'LOG_UUCP',
'LOG_WARNING',
'M_1_PI',
'M_2_PI',
'M_2_SQRTPI',
'M_E',
'M_LN10',
'M_LN2',
'M_LOG10E',
'M_LOG2E',
'M_PI',
'M_PI_2',
'M_PI_4',
'M_SQRT1_2',
'M_SQRT2',
'MON_1',
'MON_10',
'MON_11',
'MON_12',
'MON_2',
'MON_3',
'MON_4',
'MON_5',
'MON_6',
'MON_7',
'MON_8',
'MON_9',
'MON_DECIMAL_POINT',
'MON_GROUPING',
'MON_THOUSANDS_SEP',
'N_CS_PRECEDES',
'N_SEP_BY_SPACE',
'N_SIGN_POSN',
'NEGATIVE_SIGN',
'NOEXPR',
'NOSTR',
'P_CS_PRECEDES',
'P_SEP_BY_SPACE',
'P_SIGN_POSN',
'PATHINFO_BASENAME',
'PATHINFO_DIRNAME',
'PATHINFO_EXTENSION',
'PM_STR',
'POSITIVE_SIGN',
'RADIXCHAR',
'SEEK_CUR',
'SEEK_END',
'SEEK_SET',
'SORT_ASC',
'SORT_DESC',
'SORT_NUMERIC',
'SORT_REGULAR',
'SORT_STRING',
'STR_PAD_BOTH',
'STR_PAD_LEFT',
'STR_PAD_RIGHT',
'T_FMT',
'T_FMT_AMPM',
'THOUSANDS_SEP',
'THOUSEP',
'YESEXPR',
'YESSTR');

type
  THash1 = class(THashTable)
  protected
    procedure InitTable; override;
  public
    function isReserbed(const Value: WideString): boolean; override;
  end;


{ THash1 }

procedure THash1.InitTable;
var
  aa1: array of word;
  i, j: integer;
begin
  FSize := 137;{}
  FMaxLen := 1;
  SetLength(aa1, Length(ss1{}));
  for i := 0 to  Length(aa1) - 1 do begin
    aa1[i] := AnkToHash(ss1{}[i]);
    if Length(ss1{}[i]) > FMaxLen then FMaxLen := Length(ss1{}[i]);
  end;
  SetLength(FTable, FSize);
  for i := 0 to  Length(FTable) - 1 do begin
    FTable[i] := 0;
  end;
  for i := 0 to  Length(aa1) - 1 do begin
    j := aa1[i];
    while FTable[j] > 0 do begin
      inc(j);
      if j >= Length(FTable) then dec(j, Length(FTable) - 1);
    end;
    FTable[j] := i + 1;
  end;
end;

function THash1.isReserbed(const Value: WideString): boolean;
var
  i: integer;
begin
  Result := false;
  if Length(Value) > FMaxLen then exit;
  i := AnkToHash(Value);
  while FTable[i] > 0 do begin
    if ss1{}[FTable[i] - 1] = Value then begin
      Result := true;
      exit;
    end;
    inc(i);
    if i >= FSize then dec(i, FSize);
  end;
end;

type
  THash2 = class(THashTable)
  protected
    procedure InitTable; override;
  public
    function isReserbed(const Value: WideString): boolean; override;
  end;

{ THash2 }

procedure THash2.InitTable;
var
  aa1: array of word;
  i, j: integer;
begin
  FSize := 2161;{}
  FMaxLen := 1;
  SetLength(aa1, Length(ss2{}));
  for i := 0 to  Length(aa1) - 1 do begin
    aa1[i] := AnkToHash(ss2{}[i]);
    if Length(ss2{}[i]) > FMaxLen then FMaxLen := Length(ss2{}[i]);
  end;
  SetLength(FTable, FSize);
  for i := 0 to  Length(FTable) - 1 do begin
    FTable[i] := 0;
  end;
  for i := 0 to  Length(aa1) - 1 do begin
    j := aa1[i];
    while FTable[j] > 0 do begin
      inc(j);
      if j >= Length(FTable) then dec(j, Length(FTable) - 1);
    end;
    FTable[j] := i + 1;
  end;
end;

function THash2.isReserbed(const Value: WideString): boolean;
var
  i: integer;
begin
  Result := false;
  if Length(Value) > FMaxLen then exit;
  i := AnkToHash(Value);
  while FTable[i] > 0 do begin
    if ss2{}[FTable[i] - 1] = Value then begin
      Result := true;
      exit;
    end;
    inc(i);
    if i >= FSize then dec(i, FSize);
  end;
end;

type
  THash3 = class(THashTable)
  protected
    procedure InitTable; override;
  public
    function isReserbed(const Value: WideString): boolean; override;
  end;


{ THash3 }

procedure THash3.InitTable;
var
  aa1: array of word;
  i, j: integer;
begin
  FSize := 479;{}
  FMaxLen := 1;
  SetLength(aa1, Length(ss3{}));
  for i := 0 to  Length(aa1) - 1 do begin
    aa1[i] := AnkToHash(ss3{}[i]);
    if Length(ss3{}[i]) > FMaxLen then FMaxLen := Length(ss3{}[i]);
  end;
  SetLength(FTable, FSize);
  for i := 0 to  Length(FTable) - 1 do begin
    FTable[i] := 0;
  end;
  for i := 0 to  Length(aa1) - 1 do begin
    j := aa1[i];
    while FTable[j] > 0 do begin
      inc(j);
      if j >= Length(FTable) then dec(j, Length(FTable) - 1);
    end;
    FTable[j] := i + 1;
  end;
end;

function THash3.isReserbed(const Value: WideString): boolean;
var
  i: integer;
begin
  Result := false;
  if Length(Value) > FMaxLen then exit;
  i := AnkToHash(Value);
  while FTable[i] > 0 do begin
    if ss3{}[FTable[i] - 1] = Value then begin
      Result := true;
      exit;
    end;
    inc(i);
    if i >= FSize then dec(i, FSize);
  end;
end;

var
  FHash1: THash1;
  FHash2: THash2;
  FHash3: THash3;


type
  THtmlList = class(TReserveList)
  public
    function Find(const s: WideString): boolean; override;
  end;

{ THtmlList }

function THtmlList.Find(const s: WideString): boolean;
begin
  Result := FHash1.isReserbed(s);
end;

type
  THtml2List = class(TReserveList)
  public
    function Find(const s: WideString): boolean; override;
  end;

{ THtml2List }

function THtml2List.Find(const s: WideString): boolean;
begin
  Result := FHash2.isReserbed(s);
end;

type
  THtml3List = class(TReserveList)
  public
    function Find(const s: WideString): boolean; override;
  end;

{ THtml3List }

function THtml3List.Find(const s: WideString): boolean;
begin
  Result := FHash3.isReserbed(s);
end;

{ TModeHtml }

procedure TModeHtml.InitReserveList;
var
  a: array of WideString;
begin
  FReserveList := THtmlList.Create(a, true);
  FReserve2List := THtml2List.Create(a, true);
  FReserve3List := THtml3List.Create(a, true);
end;

function TModeHtml.NewParser(target: TNotePadString): TNotePadParser;
begin
  Result := TParserHtml.Create(target, self);
end;

{ TParserHtml }

constructor TParserHtml.Create(target: TNotePadString; mode: TModeHtml);
begin
  inherited Create(target, mode);
end;

procedure TParserHtml.ParseToken(AHttp: boolean);
var
  C1, C2: WideChar;
begin
  C1 := FSb[FParsePos];
  //PHP̃qAhLgI[hǂ
  //<!-- -->Rgǂ
  //html^Oǂ
  //<? ?>^Oǂ
  //<^OJnǂ
  //ankł\ɃqbgȂ
  //PHPqAhtml^OȌ
  if FParseDataString <> '' then begin
    //g[Nr̕ɂqbgĂ܂̂͂܂
    if FParsePos + Length(FParseDataString) + 1{;̕} <= FSb.StringLength then begin
      //AȂqȀIoȂ͂
      if FSb[FParsePos - 1] = #10 then begin
        if (FSb.ToString(FParsePos, Length(FParseDataString)) = FParseDataString)
        and (FSb[FParsePos + Length(FParseDataString)] = ';') then begin
          inc(FParsePos, Length(FParseDataString) - 1);
          ParseToken(toHereEnd);
          exit;
        end;
      end;
    end;
  end;
  //<!-->܂Ԃ܂ł̏
  if FToken = toCommentBrock3Start then begin
    ParseToken(toCommentBrock3);
    exit;
  end;
  if FElement = elementTag then begin
    case C1 of
      #$3E: begin
        ParseToken(toTagEnd);
        exit;
      end;
      #$2F: begin
        if FParsePos < FSb.StringLength - 1 then begin
          C2 := FSb[FParsePos + 1];
          if C2 = #$3E then begin
            inc(FParsePos);
            ParseToken(toTagEnd);
            exit;
          end;
        end;
      end;
      #$27: begin
        if FParsePos < FSb.StringLength - 1 then begin
          inc(FParsePos);
          ParseToken(toSingleQuotation);
          exit;
        end;
        ParseToken(toSingleQuotation);
        exit;
      end;
      #$22: begin
        if FParsePos < FSb.StringLength - 1 then begin
          inc(FParsePos);
          ParseToken(toDoubleQuotation);
          exit;
        end;
        ParseToken(toDoubleQuotation);
        exit;
      end;
    end;
    ParseToken(toInsideTag);
    exit;
  end;
  if FElement = elementTagQuest then begin
    if FToken = toCommentBrock1Start then begin
      ParseToken(toCommentBrock1);
      exit;
    end;
    if FToken = toHereStart then begin
      ParseToken(toHereStartWord);
      exit;
    end;
    case C1 of
      '?': begin
        if FParsePos < FSb.StringLength - 1 then begin
          C2 := FSb[FParsePos + 1];
          if C2 = '>' then begin
            inc(FParsePos);
            ParseToken(toTagQuestEnd);
            exit;
          end;
        end;
      end;
      #$27: begin
        if FParsePos < FSb.StringLength - 1 then begin
          inc(FParsePos);
          ParseToken(toSingleQuotation);
          exit;
        end;
        ParseToken(toSingleQuotation);
        exit;
      end;
      #$22: begin
        if FParsePos < FSb.StringLength - 1 then begin
          inc(FParsePos);
          ParseToken(toDoubleQuotation);
          exit;
        end;
        ParseToken(toDoubleQuotation);
        exit;
      end;
      '/': begin
        if FParsePos < FSb.StringLength - 1 then begin
          C2 := FSb[FParsePos + 1];
          if C2 = '/' then begin
            inc(FParsePos);
            ParseToken(toComment);
            exit;
          end;
          if C2 = '*' then begin
            inc(FParsePos);
            ParseToken(toCommentBrock1Start);
            exit;
          end;
        end;
      end;
      '#': begin
        ParseToken(toComment);
        exit;
      end;
      '<': begin
        if FParsePos < FSb.StringLength - 2 then begin
          C2 := FSb[FParsePos + 1];
          if C2 = '<' then begin
            C2 := FSb[FParsePos + 2];
            if C2 = '<' then begin
              inc(FParsePos, 2);
              ParseToken(toHereStart);
              exit;
            end;
          end;
        end;
      end;
      '$':begin
        if FParsePos < FSb.StringLength - 1 then begin
          C2 := FSb[FParsePos + 1];
          //if IsWordChar(C2) then begin
          if TokenFromFirstChar(C2) = toAnk then begin
            inc(FParsePos);
            ParseToken(toAnk);
            exit;
          end;
        end;
      end;
    end;//case C1
    inherited;//ParseToken(toInsideTag);
    exit;
  end;
  if C1 = '<' then begin
    if FParsePos < FSb.StringLength - 1 then begin
      C2 := FSb[FParsePos + 1];
      case C2 of
        '/': begin
          inc(FParsePos);
          ParseToken(toTagStart);
          exit;
        end;
        '?': begin
          //?݂̂̏ꍇ?php̏ꍇ̗FȂA
          //̌ɃXy[Xs鎖𔻒肵Ȃ΂ȂB
          //肾A?xmlƂɔPHPƂ낵낤B
          if (FParsePos < FSb.StringLength - 4) and
            (FSb[FParsePos + 2] = 'x') and
            (FSb[FParsePos + 3] = 'm') and
            (FSb[FParsePos + 4] = 'l') then begin
            inc(FParsePos, 4);
            ParseToken(toTagStart);
            exit;
          end;
          inc(FParsePos);
          ParseToken(toTagQuestStart);
          exit;
        end;
        '!': begin
          if FParsePos < FSb.StringLength - 3 then begin
            if (FSb[FParsePos + 2] = #$2D) and (FSb[FParsePos + 3] = #$2D) then begin
              inc(FParsePos, 3);
              ParseToken(toCommentBrock3Start);
              exit;
            end;
          end;
          inc(FParsePos);
          ParseToken(toTagStart);
          exit;
        end;
      end;
    end;
    ParseToken(toTagStart);
    exit;
  end;
  //toAnkNoReserveȂႢȂ̂ŁA
  //URLƂȂB
  if ParseTokenHttp then exit;

  if TokenFromFirstChar(C1) = toAnk then begin
    ParseToken(toAnkNoReserve);
    exit;
  end;
  if FParseDataString <> '' then begin
    case C1 of
      '$':begin
        if FParsePos < FSb.StringLength - 1 then begin
          C2 := FSb[FParsePos + 1];
          //if IsWordChar(C2) then begin
          if TokenFromFirstChar(C2) = toAnk then begin
            inc(FParsePos);
            ParseToken(toAnk);
            exit;
          end;
        end;
      end;
    end;//case C1
  end;
  inherited ParseToken(false);
end;

procedure TParserHtml.ParseToken(const C: WideChar);
begin
  case C of
    toTagStart: begin
      FToken := C;
      TagStartProc;
    end;
    toTagEnd: begin
      FToken := C;
      TagEndProc;
    end;
    toTagQuestStart: begin
      FToken := C;
      TagQuestStartProc;
    end;
    toTagQuestEnd: begin
      FToken := C;
      TagQuestEndProc;
    end;
    toSingleQuotation: begin
      FToken := C;
      if FElement = elementTagQuest then
        PhpSingleQuotSkipProc(false)
      else
        SingleQuotationHtmlProc;
    end;
    toSingleQuotSkip: begin
      FToken := toSingleQuotation;
      PhpSingleQuotSkipProc(true);
    end;
    toDoubleQuotation: begin
      FToken := C;
      if FElement = elementTagQuest then
        PhpDoubleQuotSkipProc(false)
      else
        DoubleQuotationHtmlProc;
    end;
    toDoubleQuotSkip: begin
      FToken := toDoubleQuotation;
      PhpDoubleQuotSkipProc(true);
    end;
    toCommentBrock3Start: begin
      FToken := C;
      CommentBrock3StartProc;
    end;
    toCommentBrock3: begin
      FToken := C;
      CommentBrock3Proc(-1);
    end;
    toCommentBrock3sp: begin
      FToken := toCommentBrock3;
      CommentBrock3Proc(1);
    end;
    toComment: begin
      FToken := C;
      CommentProc;
    end;
    toCommentBrock1Start: begin
      FToken := C;
      CommentBrock1StartProc;
    end;
    toCommentBrock1: begin
      FToken := C;
      CommentBrock1Proc(0);
    end;
    toCommentBrock1sp: begin
      FToken := toCommentBrock1;
      CommentBrock1Proc(1);
    end;
    toHereStart: begin
      FToken := C;
      PhpHereStartProc;
    end;
    toHereStartWord: begin
      FToken := C;
      PhpHereStartWordProc;
    end;
    toHereEnd: begin
      FToken := C;
      PhpHereEndProc;
    end;
    else inherited;
  end;
end;

initialization
  FHash1 := THash1.Create;
  FHash2 := THash2.Create;
  FHash3 := THash3.Create;
finalization
  FHash1.Free;
  FHash2.Free;
  FHash3.Free;
end.
