unit ModePerl;

interface

uses NotePadMode, NotePadString;

type
  TModePerl = class(TNotePadMode)
  protected
    procedure InitReserveList;override;
  public
    function NewParser(target: TNotePadString): TNotePadParser;override;
  end;

  TParserPerl = class(TNotePadParser)
  private
  protected
    procedure ParseToken(AHttp: boolean);overload;override;
    procedure ParseToken(const C: WideChar);overload;override;
  public
    constructor Create(target: TNotePadString; mode: TModePerl);
  end;

implementation

uses NotePadClasses, ParserConsts, SysUtils;

const
  ss1: array[0..20] of WideString =(
    'cmp',
    'do',
    'else',
    'elsif',
    'eq',
    'for',
    'foreach',
    'ge',
    'gt',
    'if',
    'le',
    'lt',
    'ne',
    'package',
    'require',
    'return',
    'sub',
    'unless',
    'until',
    'use',
    'while');

type
  THash1 = class(THashTable)
  protected
    procedure InitTable; override;
  public
    function isReserbed(const Value: WideString): boolean; override;
  end;

{ THash1 }

procedure THash1.InitTable;
var
  aa1: array of word;
  i, j: integer;
begin
  FSize := 43;{}
  FMaxLen := 1;
  SetLength(aa1, Length(ss1{}));
  for i := 0 to  Length(aa1) - 1 do begin
    aa1[i] := AnkToHash(ss1{}[i]);
    if Length(ss1{}[i]) > FMaxLen then FMaxLen := Length(ss1{}[i]);
  end;
  SetLength(FTable, FSize);
  for i := 0 to  Length(FTable) - 1 do begin
    FTable[i] := 0;
  end;
  for i := 0 to  Length(aa1) - 1 do begin
    j := aa1[i];
    while FTable[j] > 0 do begin
      inc(j);
      if j >= Length(FTable) then dec(j, Length(FTable) - 1);
    end;
    FTable[j] := i + 1;
  end;
end;

function THash1.isReserbed(const Value: WideString): boolean;
var
  i: integer;
begin
  Result := false;
  if Length(Value) > FMaxLen then exit;
  i := AnkToHash(Value);
  while FTable[i] > 0 do begin
    if ss1{}[FTable[i] - 1] = Value then begin
      Result := true;
      exit;
    end;
    inc(i);
    if i >= FSize then dec(i, FSize);
  end;
end;

var
  FHash1: THash1;

type
  TPerlList = class(TReserveList)
  public
    function Find(const s: WideString): boolean; override;
  end;

{ TPerlList }

function TPerlList.Find(const s: WideString): boolean;
begin
  Result := FHash1.isReserbed(s);
end;

{ TModePerl }

procedure TModePerl.InitReserveList;
var
  a: array of WideString;
begin
  FReserveList := TPerlList.Create(a, true);
  FReserve2List := TReserveList.Create(a, true);
  FReserve3List := TReserveList.Create(a, true);
end;

function TModePerl.NewParser(target: TNotePadString): TNotePadParser;
begin
  Result := TParserPerl.Create(target, self);
end;

{ TParserPerl }

constructor TParserPerl.Create(target: TNotePadString;
  mode: TModePerl);
begin
  inherited Create(target, mode);
end;

procedure TParserPerl.ParseToken(AHttp: boolean);
var
  C1, C2: WideChar;
begin
  C1 := FSb[FParsePos];
  if FParseDataString <> '' then begin
    //g[Nr̕ɂqbgĂ܂̂͂܂
    //PHPƈ;ŏIȂ
    if FParsePos + Length(FParseDataString) <= FSb.StringLength then begin
      //AȂqȀIoȂ͂
      if FSb[FParsePos - 1] = #10 then begin
        if FSb.ToString(FParsePos, Length(FParseDataString)) = FParseDataString then begin
          inc(FParsePos, Length(FParseDataString) - 1);
          ParseToken(toHereEnd);
          exit;
        end;
      end;
    end;
//html
    //<!-->܂Ԃ܂ł̏
    if FToken = toCommentBrock3Start then begin
      ParseToken(toCommentBrock3);
      exit;
    end;
    if FElement = elementTag then begin
      case C1 of
        #$3E: begin
          ParseToken(toTagEnd);
          exit;
        end;
        #$2F: begin
          if FParsePos < FSb.StringLength - 1 then begin
            C2 := FSb[FParsePos + 1];
            if C2 = #$3E then begin
              inc(FParsePos);
              ParseToken(toTagEnd);
              exit;
            end;
          end;
        end;
        #$27: begin
          if FParsePos < FSb.StringLength - 1 then begin
            inc(FParsePos);
            ParseToken(toSingleQuotation);
            exit;
          end;
          ParseToken(toSingleQuotation);
          exit;
        end;
        #$22: begin
          if FParsePos < FSb.StringLength - 1 then begin
            inc(FParsePos);
            ParseToken(toDoubleQuotation);
            exit;
          end;
          ParseToken(toDoubleQuotation);
          exit;
        end;
      end;
      ParseToken(toInsideTag);
      exit;
    end;
    if FElement = elementTagQuest then begin
      case C1 of
        #$3F: begin
          if FParsePos < FSb.StringLength - 1 then begin
            C2 := FSb[FParsePos + 1];
            if C2 = #$3E then begin
              inc(FParsePos);
              ParseToken(toTagQuestEnd);
              exit;
            end;
          end;
        end;
        #$27: begin
          if FParsePos < FSb.StringLength - 1 then begin
            inc(FParsePos);
            ParseToken(toSingleQuotation);
            exit;
          end;
          ParseToken(toSingleQuotation);
          exit;
        end;
        #$22: begin
          if FParsePos < FSb.StringLength - 1 then begin
            inc(FParsePos);
            ParseToken(toDoubleQuotation);
            exit;
          end;
          ParseToken(toDoubleQuotation);
          exit;
        end;
      end;
      ParseToken(toInsideTag);
      exit;
    end;
    if C1 = #$3C then begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        case C2 of
          #$2F: begin
            inc(FParsePos);
            ParseToken(toTagStart);
            exit;
          end;
          #$3F: begin
            inc(FParsePos);
            ParseToken(toTagQuestStart);
            exit;
          end;
          #$21: begin
            if FParsePos < FSb.StringLength - 3 then begin
              if (FSb[FParsePos + 2] = #$2D) and (FSb[FParsePos + 3] = #$2D) then begin
                inc(FParsePos, 3);
                ParseToken(toCommentBrock3Start);
                exit;
              end;
            end;
            inc(FParsePos);
            ParseToken(toTagStart);
            exit;
          end;
        end;
      end;
      ParseToken(toTagStart);
      exit;
    end;
    if ParseTokenHttp then exit;
    if TokenFromFirstChar(C1) = toAnk then begin
      ParseToken(toAnkNoReserve);
      exit;
    end;
//html܂
    inherited ParseToken(false);
    exit;
  end;
  if FToken = toHereStart then begin
    ParseToken(toHereStartWord);
    exit;
  end;
  if FToken = toMatchStart then begin
    ParseToken(toRegExp);
    exit;
  end;
  if FToken = toSubStart then begin
    ParseToken(toSubMatch);
    exit;
  end;
  if FToken = toSubMatch then begin
    ParseToken(toRegExp);
    exit;
  end;
  if (FLine.Start = FParsePos) then begin
    if (C1 = '=') then begin
      if FParsePos < FSb.StringLength - 5 then begin
        if (FSb[FParsePos + 1] = 'b') and
           (FSb[FParsePos + 2] = 'e') and
           (FSb[FParsePos + 3] = 'g') and
           (FSb[FParsePos + 4] = 'i') and
           (FSb[FParsePos + 5] = 'n') and
           ((FParsePos + 1 = FSb.StringLength - 5) or (TokenFromFirstChar(FSb[FParsePos + 6]) <> toAnk))
        then begin
          inc(FParsePos, 5);
          ParseToken(toPodComment);
          exit;
        end;
        if (FSb[FParsePos + 1] = 'h') and
           (FSb[FParsePos + 2] = 'e') and
           (FSb[FParsePos + 3] = 'a') and
           (FSb[FParsePos + 4] = 'd') and
           ((FSb[FParsePos + 5] = '1') or (FSb[FParsePos + 5] = '2')) and
           ((FParsePos + 1 = FSb.StringLength - 5) or (TokenFromFirstChar(FSb[FParsePos + 6]) <> toAnk))
        then begin
          inc(FParsePos, 5);
          ParseToken(toPodComment);
          exit;
        end;
      end;
      if FParsePos < FSb.StringLength - 4 then begin
        if (FSb[FParsePos + 1] = 'b') and
           (FSb[FParsePos + 2] = 'a') and
           (FSb[FParsePos + 3] = 'c') and
           (FSb[FParsePos + 4] = 'k') and
           ((FParsePos + 1 = FSb.StringLength - 4) or (TokenFromFirstChar(FSb[FParsePos + 5]) <> toAnk))
        then begin
          inc(FParsePos, 4);
          ParseToken(toPodComment);
          exit;
        end;
        if (FSb[FParsePos + 1] = 'i') and
           (FSb[FParsePos + 2] = 't') and
           (FSb[FParsePos + 3] = 'e') and
           (FSb[FParsePos + 4] = 'm') and
           ((FParsePos + 1 = FSb.StringLength - 4) or (TokenFromFirstChar(FSb[FParsePos + 5]) <> toAnk))
        then begin
          inc(FParsePos, 4);
          ParseToken(toPodComment);
          exit;
        end;
        if (FSb[FParsePos + 1] = 'o') and
           (FSb[FParsePos + 2] = 'v') and
           (FSb[FParsePos + 3] = 'e') and
           (FSb[FParsePos + 4] = 'r') and
           ((FParsePos + 1 = FSb.StringLength - 4) or (TokenFromFirstChar(FSb[FParsePos + 5]) <> toAnk))
        then begin
          inc(FParsePos, 4);
          ParseToken(toPodComment);
          exit;
        end;
      end;
      if FParsePos < FSb.StringLength - 3 then begin
        if (FSb[FParsePos + 1] = 'c') and
           (FSb[FParsePos + 2] = 'u') and
           (FSb[FParsePos + 3] = 't') and
           ((FParsePos + 1 = FSb.StringLength - 3) or (TokenFromFirstChar(FSb[FParsePos + 4]) <> toAnk))
        then begin
          inc(FParsePos, 3);
          ParseToken(toPodCommentEnd);
          exit;
        end;
        if (FSb[FParsePos + 1] = 'p') and
           (FSb[FParsePos + 2] = 'o') and
           (FSb[FParsePos + 3] = 'd') and
           ((FParsePos + 1 = FSb.StringLength - 3) or (TokenFromFirstChar(FSb[FParsePos + 4]) <> toAnk))
        then begin
          inc(FParsePos, 3);
          ParseToken(toPodComment);
          exit;
        end;
        if (FSb[FParsePos + 1] = 'f') and
           (FSb[FParsePos + 2] = 'o') and
           (FSb[FParsePos + 3] = 'r') and
           ((FParsePos + 1 = FSb.StringLength - 3) or (TokenFromFirstChar(FSb[FParsePos + 4]) <> toAnk))
        then begin
          inc(FParsePos, 3);
          ParseToken(toPodComment);
          exit;
        end;
        if (FSb[FParsePos + 1] = 'e') and
           (FSb[FParsePos + 2] = 'n') and
           (FSb[FParsePos + 3] = 'd') and
           ((FParsePos + 1 = FSb.StringLength - 3) or (TokenFromFirstChar(FSb[FParsePos + 4]) <> toAnk))
        then begin
          inc(FParsePos, 3);
          ParseToken(toPodComment);
          exit;
        end;
      end;
    end;
    //słApodȊO
    if FToken = toPodComment then begin
      ParseToken(toPodComment);
      exit;
    end;
  end;
  case C1 of
    '(', '+' .. ',', '=', '~': begin
      ParseToken(toSymbolNoDiv);
      exit;
    end;
    #$27: begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if C2 <> #10 then begin
          inc(FParsePos);
          ParseToken(toSingleQuotation);
          exit;
        end;
      end;
      ParseToken(toSingleQuotation);
      exit;
    end;
    #$22: begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if C2 <> #10 then begin
          inc(FParsePos);
          ParseToken(toDoubleQuotation);
          exit;
        end;
      end;
      ParseToken(toDoubleQuotation);
      exit;
    end;
    #$23: begin
      ParseToken(toComment);
      exit;
    end;
    '0': begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if (C2 = 'X') or (C2 = 'x') then begin
          inc(FParsePos);
          ParseToken(toHex);
          exit;
        end;
      end;
    end;
    #$24:begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
          //if IsWordChar(C2) then begin
          if TokenFromFirstChar(C2) = toAnk then begin
          inc(FParsePos);
          ParseToken(toAnk);
          exit;
        end;
        if C2 = #$23 then begin
          inc(FParsePos);
          ParseToken(toAnk);
          exit;
        end;
      end;
    end;
    #$26, #$40, #$2A, #$25: begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
          //if IsWordChar(C2) then begin
          if TokenFromFirstChar(C2) = toAnk then begin
          inc(FParsePos);
          ParseToken(toAnk);
          exit;
        end;
      end;
    end;
    '<': begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if C2 = '<' then begin
          inc(FParsePos);
          ParseToken(toHereStart);
          exit;
        end;
      end;
    end;
    '/': begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if (FPrevToken = toSymbolNoDiv) and (C2 <> #10) then begin
          inc(FParsePos);
          ParseToken(toRegExp);
          exit;
        end;
      end;
      if FPrevToken = toSymbolNoDiv then begin
        ParseToken(toRegExp);
        exit;
      end;
    end;
    'm': begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if C2 = '/' then begin
          inc(FParsePos);
          ParseToken(toMatchStart);
          exit;
        end;
      end;
    end;
    's': begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if C2 = '/' then begin
          inc(FParsePos);
          ParseToken(toSubStart);
          exit;
        end;
      end;
    end;
  end;//case
  inherited ParseToken(false);
end;

procedure TParserPerl.ParseToken(const C: WideChar);
begin
  case C of
    toSingleQuotation: begin
      FToken := C;
      if FElement = #0 then
        SingleQuotSkipProc(false)
      else
        SingleQuotationHtmlProc;
    end;
    toSingleQuotSkip: begin
      FToken := toSingleQuotation;
      SingleQuotSkipProc(true);
    end;
    toDoubleQuotation: begin
      FToken := C;
      if FElement = #0 then
        DoubleQuotSkipProc(false)
      else
        DoubleQuotationHtmlProc;
    end;
    toDoubleQuotSkip: begin
      FToken := toDoubleQuotation;
      DoubleQuotSkipProc(true);
    end;
    toComment: begin
      FToken := C;
      CommentProc;
    end;
    toHex: begin
      FToken := C;
      HexProc;
    end;
    toHereStart: begin
      FToken := C;
      PerlHereStartProc;
    end;
    toHereStartWord: begin
      FToken := C;
      PerlHereStartWordProc;
    end;
    toHereEnd: begin
      FToken := C;
      PerlHereEndProc;
    end;
    toTagStart: begin
      FToken := C;
      TagStartProc;
    end;
    toTagEnd: begin
      FToken := C;
      TagEndProc;
    end;
    toTagQuestStart: begin
      FToken := C;
      TagQuestStartProc;
    end;
    toTagQuestEnd: begin
      FToken := C;
      TagQuestEndProc;
    end;
    toCommentBrock3Start: begin
      FToken := C;
      CommentBrock3StartProc;
    end;
    toCommentBrock3: begin
      FToken := C;
      CommentBrock3Proc(-1);
    end;
    toCommentBrock3sp: begin
      FToken := toCommentBrock3;
      CommentBrock3Proc(1);
    end;
    toRegExp: begin
      FToken := C;
      RegExpSkipProc(false);
    end;
    toRegExpSkip: begin
      FToken := toRegExp;
      RegExpSkipProc(true);
    end;
    toMatchStart: begin
      FToken := C;
      MatchStartProc;
    end;
    toSubStart: begin
      FToken := C;
      SubStartProc;
    end;
    toSubMatch: begin
      FToken := C;
      SubMatchSkipProc(false);
    end;
    toSubMatchSkip: begin
      FToken := toSubMatch;
      SubMatchSkipProc(true);
    end;
    toPodComment: begin
      FToken := toPodComment;
      PodCommentProc;
    end;
    else inherited;
  end;
end;

initialization
  FHash1 := THash1.Create;
finalization
  FHash1.Free;
end.

//http://allabout.co.jp/career/cgiperl/closeup/CU20030307A/index.htm
//0x0X
//啶Ə͋ʂ
//VONH[g̓GXP[vȂA\'ł\\'ŕȂXLbvŉ
//@q/ / Ƃ qq/ / Ƃ q{ } Ƃ܂łł񂾁H
//qAhLg
//http://infosys.gsid.nagoya-u.ac.jp/~ohna/perl_lesson/heredoc.html
//$var = << "EOM";  #NH[gň͂߂΁C<< ̌ɃXy[XĂ悢B
//EOM               #ɂ̓NH[g͕tȂB

//XbVŎn܂鐳K\ubNǂ邩H
//ςȃR[e[Vǂ邩
