unit ModePython;

interface

uses NotePadMode, NotePadString;

type
  TModePython = class(TNotePadMode)
  protected
    procedure InitReserveList;override;
  public
    function NewParser(target: TNotePadString): TNotePadParser;override;
  end;

  TParserPython = class(TNotePadParser)
  private
  protected
    procedure ParseToken(AHttp: boolean);overload;override;
    procedure ParseToken(const C: WideChar);overload;override;
  public
    constructor Create(target: TNotePadString; mode: TModePython);
  end;

implementation

uses NotePadClasses, ParserConsts;

const
  ss1: array[0..27] of WideString =(
    'access',
    'and',
    'break',
    'class',
    'continue',
    'def',
    'del',
    'elif',
    'else',
    'expect',
    'exec',
    'finally',
    'for',
    'form',
    'global',
    'if',
    'import',
    'in',
    'is',
    'lambda',
    'not',
    'or',
    'pass',
    'print',
    'raise',
    'return',
    'try',
    'while');

type
  THash1 = class(THashTable)
  protected
    procedure InitTable; override;
  public
    function isReserbed(const Value: WideString): boolean; override;
  end;

{ THash1 }

procedure THash1.InitTable;
var
  aa1: array of word;
  i, j: integer;
begin
  FSize := 59;{}
  FMaxLen := 1;
  SetLength(aa1, Length(ss1{}));
  for i := 0 to  Length(aa1) - 1 do begin
    aa1[i] := AnkToHash(ss1{}[i]);
    if Length(ss1{}[i]) > FMaxLen then FMaxLen := Length(ss1{}[i]);
  end;
  SetLength(FTable, FSize);
  for i := 0 to  Length(FTable) - 1 do begin
    FTable[i] := 0;
  end;
  for i := 0 to  Length(aa1) - 1 do begin
    j := aa1[i];
    while FTable[j] > 0 do begin
      inc(j);
      if j >= Length(FTable) then dec(j, Length(FTable) - 1);
    end;
    FTable[j] := i + 1;
  end;
end;

function THash1.isReserbed(const Value: WideString): boolean;
var
  i: integer;
begin
  Result := false;
  if Length(Value) > FMaxLen then exit;
  i := AnkToHash(Value);
  while FTable[i] > 0 do begin
    if ss1{}[FTable[i] - 1] = Value then begin
      Result := true;
      exit;
    end;
    inc(i);
    if i >= FSize then dec(i, FSize);
  end;
end;

var
  FHash1: THash1;

type
  TPythonList = class(TReserveList)
  public
    function Find(const s: WideString): boolean; override;
  end;

{ TPythonList }

function TPythonList.Find(const s: WideString): boolean;
begin
  Result := FHash1.isReserbed(s);
end;

{ TModePython }

procedure TModePython.InitReserveList;
var
  a: array of WideString;
begin
  FReserveList := TPythonList.Create(a, true);
  FReserve2List := TReserveList.Create(a, true);
  FReserve3List := TReserveList.Create(a, true);
end;

function TModePython.NewParser(target: TNotePadString): TNotePadParser;
begin
  Result := TParserPython.Create(target, self);
end;

{ TParserPython }

constructor TParserPython.Create(target: TNotePadString;
  mode: TModePython);
begin
  inherited Create(target, mode);
end;

procedure TParserPython.ParseToken(AHttp: boolean);
var
  C2: WideChar;
begin
  case FSb[FParsePos] of
    #$27: begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if C2 <> #10 then begin
          inc(FParsePos);
          ParseToken(toSingleQuotation);
          exit;
        end;
      end;
      ParseToken(toSingleQuotation);
      exit;
    end;
    #$22: begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if C2 <> #10 then begin
          inc(FParsePos);
          ParseToken(toDoubleQuotation);
          exit;
        end;
      end;
      ParseToken(toDoubleQuotation);
      exit;
    end;
    #$23: begin
      ParseToken(toComment);
      exit;
    end;
    #$30: begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if (C2 = #$58) or (C2 = #$78) then begin
          inc(FParsePos);
          ParseToken(toHex);
          exit;
        end;
      end;
    end;
  end;//case
  inherited ParseToken(false);
end;

procedure TParserPython.ParseToken(const C: WideChar);
begin
  case C of
    toSingleQuotation: begin
      FToken := C;
      SingleQuotSkipProc(false);
    end;
    toSingleQuotSkip: begin
      FToken := toSingleQuotation;
      SingleQuotSkipProc(true);
    end;
    toDoubleQuotation: begin
      FToken := C;
      DoubleQuotSkipProc(false);
    end;
    toDoubleQuotSkip: begin
      FToken := toDoubleQuotation;
      DoubleQuotSkipProc(true);
    end;
    toComment: begin
      FToken := C;
      CommentProc;
    end;
    toHex: begin
      FToken := C;
      HexProc;
    end;
    else inherited;
  end;
end;

initialization
  FHash1 := THash1.Create;
finalization
  FHash1.Free;
end.
