unit ModeVBScript;

interface

uses Windows, NotePadMode, NotePadString;

type
  TModeVBScript = class(TNotePadMode)
  protected
    procedure InitReserveList;override;
  public
    function NewParser(target: TNotePadString): TNotePadParser;override;
  end;

  TParserVBScript = class(TNotePadParser)
  private
  protected
    procedure ParseToken(AHttp: boolean);overload;override;
    procedure ParseToken(const C: WideChar);overload;override;
  public
    constructor Create(target: TNotePadString; mode: TModeVBScript);
  end;

implementation

uses NotePadClasses, ParserConsts;

const
  ss1: array[0..160] of WideString =(
    'abs',
    'and',
    'array',
    'asc',
    'ascb',
    'ascw',
    'atn',
    'byref',
    'byval',
    'cbool',
    'cbyte',
    'ccur',
    'cdate',
    'cdbl',
    'cint',
    'class',
    'clng',
    'csng',
    'cstr',
    'call',
    'case',
    'chr',
    'chrb',
    'chrw',
    'const',
    'cos',
    'createobject',
    'date',
    'dateadd',
    'datediff',
    'datepart',
    'dateserial',
    'datevalue',
    'day',
    'dim',
    'do',
    'each',
    'else',
    'elseif',
    'empty',
    'end',
    'eqv',
    'erase',
    'err',
    'error',
    'escape',
    'eval',
    'execute',
    'executeglobal',
    'exit',
    'exp',
    'explicit',
    'false',
    'filter',
    'fix',
    'for',
    'formatcurrency',
    'formatdatetime',
    'formatnumber',
    'formatpercent',
    'function',
    'getlocale',
    'getobject',
    'getref',
    'hex',
    'hour',
    'if',
    'imp',
    'in',
    'instr',
    'instrb',
    'instrrev',
    'inputbox',
    'int',
    'is',
    'isarray',
    'isdate',
    'isempty',
    'isnull',
    'isnumeric',
    'isobject',
    'join',
    'lbound',
    'lcase',
    'ltrim',
    'left',
    'leftb',
    'len',
    'lenb',
    'loadpicture',
    'log',
    'loop',
    'mid',
    'midb',
    'minute',
    'mod',
    'month',
    'monthname',
    'msgbox',
    'next',
    'not',
    'nothing',
    'new',
    'now',
    'null',
    'oct',
    'on',
    'option',
    'or',
    'private',
    'public',
    'rtrim',
    'randomize',
    'redim',
//    'rem',
    'replace',
    'rgb',
    'right',
    'rightb',
    'rnd',
    'round',
    'scriptengine',
    'scriptenginebuildversion',
    'scriptenginemajorversion',
    'scriptengineminorversion',
    'second',
    'select',
    'set',
    'setlocale',
    'sgn',
    'sin',
    'space',
    'split',
    'sqr',
    'step',
    'stop',
    'strcomp',
    'strreverse',
    'string',
    'sub',
    'tan',
    'then',
    'time',
    'timer',
    'timeserial',
    'timevalue',
    'to',
    'trim',
    'true',
    'typename',
    'ubound',
    'ucase',
    'unescape',
    'until',
    'vartype',
    'wend',
    'weekday',
    'weekdayname',
    'while',
    'with',
    'xor',
    'year');

type
  THash1 = class(THashTable)
  protected
    procedure InitTable; override;
  public
    function isReserbed(const Value: WideString): boolean; override;
  end;

{ THash1 }

procedure THash1.InitTable;
var
  aa1: array of word;
  i, j: integer;
begin
  FSize := 331;{}
  FMaxLen := 1;
  SetLength(aa1, Length(ss1{}));
  for i := 0 to  Length(aa1) - 1 do begin
    aa1[i] := AnkToHash(ss1{}[i]);
    if Length(ss1{}[i]) > FMaxLen then FMaxLen := Length(ss1{}[i]);
  end;
  SetLength(FTable, FSize);
  for i := 0 to  Length(FTable) - 1 do begin
    FTable[i] := 0;
  end;
  for i := 0 to  Length(aa1) - 1 do begin
    j := aa1[i];
    while FTable[j] > 0 do begin
      inc(j);
      if j >= Length(FTable) then dec(j, Length(FTable) - 1);
    end;
    FTable[j] := i + 1;
  end;
end;

function THash1.isReserbed(const Value: WideString): boolean;
var
  i: integer;
begin
  Result := false;
  if Length(Value) > FMaxLen then exit;
  i := AnkToHash(Value);
  while FTable[i] > 0 do begin
    if CharLowerW(PWChar(ss1{}[FTable[i] - 1])) = Value then begin
      Result := true;
      exit;
    end;
    inc(i);
    if i >= FSize then dec(i, FSize);
  end;
end;

var
  FHash1: THash1;

type
  TVBScriptList = class(TReserveList)
  public
    function Find(const s: WideString): boolean; override;
  end;

{ TVBScriptList }

function TVBScriptList.Find(const s: WideString): boolean;
begin
  Result := FHash1.isReserbed(CharLowerW(PWChar(s)));
end;

{ TModeVBScript }

procedure TModeVBScript.InitReserveList;
var
  a: array of WideString;
begin
  FReserveList := TVBScriptList.Create(a, false);
  FReserve2List := TReserveList.Create(a, true);
  FReserve3List := TReserveList.Create(a, true);
end;

function TModeVBScript.NewParser(target: TNotePadString): TNotePadParser;
begin
  Result := TParserVBScript.Create(target, self);
end;

{ TParserVBScript }

constructor TParserVBScript.Create(target: TNotePadString;
  mode: TModeVBScript);
begin
  inherited Create(target, mode);
end;

procedure TParserVBScript.ParseToken(AHttp: boolean);
var
  C2, C3, C4: WideChar;
begin
  case FSb[FParsePos] of
    #$22: begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if C2 <> #10 then begin
          inc(FParsePos);
          ParseToken(toDoubleQuotation);
          exit;
        end;
      end;
      ParseToken(toDoubleQuotation);
      exit;
    end;
    #$27: begin
      ParseToken(toComment);
      exit;
    end;
    '#': begin
      ParseToken(toDirective);
      exit;
    end;
    #$26: begin
      if FParsePos < FSb.StringLength - 1 then begin
        C2 := FSb[FParsePos + 1];
        if (C2 = #$48) or (C2 = #$68) then begin
          inc(FParsePos);
          ParseToken(toHex);
          exit;
        end;
      end;
    end;
    #$52, #$72: begin
      if FParsePos < FSb.StringLength - 2 then begin
        C2 := FSb[FParsePos + 1];
        C3 := FSb[FParsePos + 2];
        if ((C2 = WideChar('E')) or (C2 = WideChar('e')))
           and ((C3 = WideChar('M')) or (C3 = WideChar('m'))) then begin
          if FParsePos < FSb.StringLength - 3 then begin
            C4 := FSb[FParsePos + 3];
            if (C4 >= WideChar('0')) and (C4 <= WideChar('z')) and
              ((C4 >= WideChar('a')) or (C4 = WideChar('_')) or
              ((C4 <= WideChar('Z'))and((C4 >= WideChar('A')) or (C4 <= WideChar('9'))))) then begin
              inherited;
              exit;
            end;
          end;
          inc(FParsePos, 2);
          ParseToken(toComment);
          exit;
        end;
      end;
    end;
  end;//case
  inherited ParseToken(false);
end;

procedure TParserVBScript.ParseToken(const C: WideChar);
begin
  case C of
    toDoubleQuotation: begin
      FToken := C;
      DoubleQuotationProc;
    end;
    toComment: begin
      FToken := C;
      CommentProc;
    end;
    toDirective: begin
      FToken := C;
      CommentProc;
    end;
    toHex: begin
      FToken := C;
      HexProc;
    end;
    else inherited;
  end;
end;

initialization
  FHash1 := THash1.Create;
finalization
  FHash1.Free;
end.
