unit NotePadClasses;

interface

uses Windows, PointerList;

type
  TLineEnd = (leNone, leWrap{, leCR}, leLF{, leCRLF});
  TLineEnds = (lsCR, lsLF, lsCRLF);

  TCaretPosition = record
    Position: integer;
    X: integer;
    Y: integer;
  end;

  PLineData = ^TLineData;
  TLineData = record
    NeedWrap: boolean;
    LineEnd: TLineEnd;
    Start: integer;
    Length: integer;
    EndToken: WideChar;
    EndElement: WideChar;
    ParseDataString: WideString;
    PrevToken: WideChar;
  end;

  TReserveList = class
  private
    FCaseSensitive: boolean;
  public
    FList: array of WideString;
    constructor Create(const ss: array of WideString; caseSensitive: boolean);
    function Find(const s: WideString): boolean; virtual;
    procedure Add(const s: WideString);
  end;

  PReconvertString = ^TReconvertString;
  tagRECONVERTSTRING = packed record
    dwSize: Cardinal;
    dwVersion: Cardinal;
    dwStrLen: Cardinal;
    dwStrOffset: Cardinal;
    dwCompStrLen: Cardinal;
    dwCompStrOffset: Cardinal;
    dwTargetStrLen: Cardinal;
    dwTargetStrOffset: Cardinal;
  end;
  TReconvertString = tagRECONVERTSTRING;

type
  TLineList = class
  private
    FList: TPointerList;
    FTrash: TPointerList;
    function GetCount: integer;
    function Get(Index: Integer): PLineData;
    procedure Put(Index: Integer; const data: PLineData);
  public
    constructor Create(size : integer);
    destructor Destroy; override;
    procedure Clear;
    procedure Add(const data: TLineData);
    procedure RemoveRange(index, Len: integer);
    procedure Insert(index: integer; const data: TLineData);
    property Count :integer read GetCount;
    property Lines[Index: Integer]: PLineData read Get write Put; default;
  end;

type
  AInteger = array of integer;
  PUndoData = ^TUndoData;
  TUndoData = record
    Position: integer;
    SelStartBefore: integer;
    SelEndBefore: integer;
    SelStartAfter: integer;
    SelEndAfter: integer;
    Deleted: WideString;
    Added: WideString;
    MarksBefore: AInteger;
    MarksAfter: AInteger;
    LineEndBefore: TLineEnds;
    LineEndAfter: TLineEnds;
  end;

  TUndoList = class
  private
    FUndoData: TPointerList;
    FRedoData: TPointerList;
    procedure ClearUndo;
    procedure ClearRedo;
    function GetCanUndo: boolean;
    function GetCanRedo: boolean;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    procedure Add(const data: TUndoData);
    function GetLastUndo : TUndoData;
    function GetLastRedo : TUndoData;
    procedure Undo;
    procedure Redo;
    property CanUndo: boolean read GetCanUndo;
    property CanRedo: boolean read GetCanRedo;
  end;

type
  THashTable = class
  private
  protected
    FSize: integer;
    FTable: array of Word;
    FMaxLen: integer;
    function AnkToHash(const Value: WideString): Word;
    procedure InitTable; virtual; abstract;
  public
    constructor Create;
    function isReserbed(const Value: WideString): boolean; virtual; abstract;
  end;

implementation

uses SysUtils;

{ TReserveList }

procedure TReserveList.Add(const s: WideString);
begin
  if Find(s) then exit;
  SetLength(FList, Length(FList) + 1);
  FList[Length(FList) - 1] := s;
end;

constructor TReserveList.Create(const ss: array of WideString;
  caseSensitive: boolean);
var
  i: integer;
begin
//ÓIzƈقȂCRs[ICg͓Izɂ͍̗pĂ炸C
//Iz񂪏ݎɎIɃRs[邱Ƃ͂܂B
  SetLength(FList, Length(ss));
  for i := 0 to Length(ss) - 1 do FList[i] := ss[i];
  FCaseSensitive := caseSensitive;
end;

function TReserveList.Find(const s: WideString): boolean;
var
  i: integer;
  ws: WideString;
begin
  if not FCaseSensitive then begin
    ws := CharLowerW(PWChar(s));
    for i := 0 to Length(FList) -1 do
      if CharLowerW(PWChar(FList[i])) = ws then begin
        Result := true;
        exit;
    end;
  end else begin
    for i := 0 to Length(FList) -1 do
      if FList[i] = s then begin
        Result := true;
        exit;
    end;
  end;
  Result := false;
end;

{ TLineList }

procedure TLineList.Add(const data: TLineData);
var
  AData: PLineData;
begin
  if FTrash.Count > 0 then begin
    AData := FTrash[FTrash.Count - 1];
    FTrash.Remove(FTrash.Count - 1, 1);
  end else
    New(AData);
  AData^ := data;
  FList.Add(AData);
end;

procedure TLineList.Clear;
var
  i: integer;
begin
  for i := 0 to FList.Count - 1 do begin
    FTrash.Add(FList[i]);
  end;
  FList.Clear;
end;

constructor TLineList.Create(size : integer);
var
  AData: PLineData;
  i: integer;
begin
  FList := TPointerList.Create(size);
  FTrash := TPointerList.Create(size);
  for i := 1 to size do begin
    New(AData);
    FTrash.Add(AData);
  end;
end;

destructor TLineList.Destroy;
var
  i: integer;
  AData: PLineData;
begin
  for i := 0 to FList.Count - 1 do begin
    AData := FList[i];
    Dispose(AData);
  end;
  FList.Free;
  for i := 0 to FTrash.Count - 1 do begin
    AData := FTrash[i];
    Dispose(AData);
  end;
  FTrash.Free;
  inherited;
end;

function TLineList.Get(Index: Integer): PLineData;
begin
  Result := PLineData(FList[Index]);
end;

function TLineList.GetCount: integer;
begin
  result := FList.Count;
end;

procedure TLineList.Insert(index: integer; const data: TLineData);
var
  AData: PLineData;
begin
  if FTrash.Count > 0 then begin
    AData := FTrash[FTrash.Count - 1];
    FTrash.Remove(FTrash.Count - 1, 1);
  end else
    New(AData);
  AData^ := data;
  FList.Insert(index, AData);
end;

procedure TLineList.Put(Index: Integer; const data: PLineData);
begin
  PLineData(FList[Index])^ := data^;
end;

procedure TLineList.RemoveRange(index, Len: integer);
var
  i: integer;
begin

  for i := index to index + Len - 1 do begin
    FTrash.Add(FList[i]);
  end;
  FList.Remove(index, Len);
end;

{ TUndoList }

procedure TUndoList.Add(const data: TUndoData);
var
  AData: PUndoData;
begin
  New(AData);
  AData^ := data;
  AData.MarksBefore := Copy(data.MarksBefore);
  AData.MarksAfter := Copy(data.MarksAfter);
  FUndoData.Add(AData);
  ClearRedo;
end;

procedure TUndoList.Clear;
begin
  ClearUndo;
  ClearRedo;
end;

procedure TUndoList.ClearRedo;
var
  i: integer;
  AData: PUndoData;
begin
  for i := 0 to FRedoData.Count - 1 do begin
    AData := FRedoData[i];
    Dispose(AData);
  end;
  FRedoData.Clear;
end;

procedure TUndoList.ClearUndo;
var
  i: integer;
  AData: PUndoData;
begin
  for i := 0 to FUndoData.Count - 1 do begin
    AData := FUndoData[i];
    Dispose(AData);
  end;
  FUndoData.Clear;
end;

constructor TUndoList.Create;
begin
  FUndoData := TPointerList.Create(0);
  FRedoData := TPointerList.Create(0);
  Clear;
end;

destructor TUndoList.Destroy;
begin
  Clear;
  FUndoData.Free;
  FRedoData.Free;
  inherited;
end;

function TUndoList.GetCanRedo: boolean;
begin
  result := FRedoData.Count > 0;
end;

function TUndoList.GetCanUndo: boolean;
begin
  result := FUndoData.Count > 0;
end;

function TUndoList.GetLastRedo: TUndoData;
begin
  Result := PUndoData(FRedoData[FRedoData.Count -1])^;
end;

function TUndoList.GetLastUndo: TUndoData;
begin
  Result := PUndoData(FundoData[FUndoData.Count -1])^;
end;

procedure TUndoList.Redo;
begin
  FUndoData.Add(FRedoData[FRedoData.Count - 1]);
  FRedoData.Remove(FRedoData.Count - 1, 1);
end;

procedure TUndoList.Undo;
begin
  FRedoData.Add(FUndoData[FUndoData.Count - 1]);
  FUndoData.Remove(FUndoData.Count - 1, 1);
end;

{ THashTable }

function THashTable.AnkToHash(const Value: WideString): Word;
const
  AnkToByte: array[$20..$7A] of byte = (
     0, 0, 0, 0,38, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     1, 2, 3, 4, 5, 6, 7, 8, 9,10, 0, 0, 0, 0, 0, 0,
     0,11,12,13,14,15,16,17,18,19,21,21,22,23,24,25,
    26,27,28,29,30,31,32,33,34,35,36, 0, 0, 0, 0,37,
     0,11,12,13,14,15,16,17,18,19,21,21,22,23,24,25,
    26,27,28,29,30,31,32,33,34,35,36
  );

var
  i: integer;
  sum: int64;
begin
  sum := 0;
  for i := 1 to Length(Value) do begin
    sum := sum + (int64(AnkToByte[Word(Value[i])]) shl (6 * (i mod 9)));
  end;
  Result := integer(sum mod int64(FSize));
end;

constructor THashTable.Create;
begin
  inherited;
  InitTable;
end;

end.
