unit NotePadMode;

interface

uses NotePadString, NotePadClasses, StringBuffer;

type
  TNotePadParser = class;

  TNotePadMode = class
  private
    function GetExtCount: integer;
    function GetExts(Index: integer): WideString;
  protected
    FFileExtList: TReserveList;
    FReserveList, FReserve2List, FReserve3List: TReserveList;
    //vpeB
    function GetFileExts: WideString;
    procedure SetFileExts(const Value: WideString);
    procedure InitReserveList;virtual;abstract;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    function GetDefExt: WideString;
    function NewParser(target: TNotePadString): TNotePadParser;virtual;abstract;
    function HasExt(FileExt: WideString): boolean;
    //vpeB
    property FileExts: WideString read GetFileExts write SetFileExts;
    property ExtCount: integer read GetExtCount;
    property Exts[Index: integer]: WideString read GetExts;
  end;

  TNotePadParser = class
  private
    procedure ParseLine(num: integer);     
  protected
    FNotePadMode: TNotePadMode;
    FNotePadString: TNotePadString;
    FSb: TStringBuffer;
    FParsePos: integer;
    FParseEnd: integer;
    FLine: PLineData;
    FToken: WideChar;
    FElement: WideChar;
    FParseDataString: WideString;
    FPrevToken: WideChar;
    procedure ParseToken(AHttp: boolean);overload;virtual;
    procedure ParseToken(const C: WideChar);overload;virtual;
    function ParseTokenHttp: boolean;virtual;
    function IsFirstCharOfLine: boolean;virtual;
    function IsVeryFirstCharOfLine: boolean;virtual;
    procedure NormalProc;
    procedure SymbolProc;
    procedure IntegerProc;
    procedure AnkProc;
    procedure Ank2Proc;
    procedure Ank3Proc;
    procedure DBSymbolProc;
    procedure DBIntProc;
    procedure DBAlphProc;
    procedure DBHiraProc;
    procedure DBKanaProc;
    procedure DBKanjiProc;
    procedure SBKanaProc;
    procedure SBKanaSymbolProc;
    procedure UrlStartProc;
    procedure FtpStartProc;
    procedure HttpsStartProc;
    procedure RtspStartProc;
    procedure PnmStartProc;
    procedure MmsStartProc;
    procedure NewsStartProc;
    procedure NntpStartProc;
    procedure UrlProc;
    procedure SingleQuotationProc;
    procedure DoubleQuotationProc;
    procedure CommentBrock0Proc;
    procedure CommentProc;
    procedure CommentBrock1StartProc;
    procedure CommentBrock1Proc(splitHit: integer);
    procedure CommentBrock2StartProc;
    procedure CommentBrock2Proc(splitHit: integer);
    procedure CommentBrock3StartProc;
    procedure CommentBrock3Proc(splitHit: integer);
    procedure HexProc;
    procedure TagStartProc;
    procedure TagEndProc;
    procedure SingleQuotSkipProc(skip: boolean);
    procedure DoubleQuotSkipProc(skip: boolean);
    procedure AtDoubleQuotStartProc;
    procedure AtDoubleQuotProc;
    procedure TagQuestStartProc;
    procedure TagQuestEndProc;
    procedure PerlHereStartProc;
    procedure PerlHereStartWordProc;
    procedure PerlHereEndProc;
    procedure RegExpSkipProc(skip: boolean);
    procedure MatchStartProc;
    procedure SubStartProc;
    procedure SubMatchSkipProc(skip: boolean);
    procedure SingleQuotationHtmlProc;
    procedure DoubleQuotationHtmlProc;
    procedure PhpHereStartProc;
    procedure PhpHereStartWordProc;
    procedure PhpHereEndProc;
    procedure PhpSingleQuotSkipProc(skip: boolean);
    procedure PhpDoubleQuotSkipProc(skip: boolean);
    procedure PodCommentProc;
  public
    constructor Create(target: TNotePadString; mode: TNotePadMode);
    function ParseRange(start1, end1: integer): integer;
    procedure ParseLineColor(num: integer);
  end;

implementation

uses ParserConsts, NotePadConsts;

{ TNotePadMode }

constructor TNotePadMode.Create;
var
  ss: array of WideString;
begin
  SetLength(ss, 0);
  FFileExtList := TReserveList.Create(ss, false);
  //FReserveList := TReserveList.Create(ss, true);
  InitReserveList;
end;

destructor TNotePadMode.Destroy;
begin
  FFileExtList.Free;
  FReserveList.Free;
  FReserve2List.Free;
  FReserve3List.Free;
  inherited;
end;

function TNotePadMode.GetDefExt: WideString;
begin
  if Length(FFileExtList.FList) = 0 then Result := '' else begin
    Result := FFileExtList.FList[0];
    if Result[1] = '.' then Delete(Result,1, 1);
  end;
end;

function TNotePadMode.GetExtCount: integer;
begin
  Result := Length(FFileExtList.FList);
end;

function TNotePadMode.GetExts(Index: integer): WideString;
begin
  Result := FFileExtList.FList[Index];
end;

function TNotePadMode.GetFileExts: WideString;
var
  i: integer;
begin
  Result := '';
  for i := 0 to Length(FFileExtList.FList) - 1 do begin
    if Result <> '' then Result := Result + #$3B;
    Result := Result + FFileExtList.FList[i]
  end;
end;

function TNotePadMode.HasExt(FileExt: WideString): boolean;
begin
  Result := FFileExtList.Find(FileExt);
end;

procedure TNotePadMode.SetFileExts(const Value: WideString);
var
  i: integer;
  ss: array of WideString;
begin
  SetLength(ss, 1);
  for i := 1 to Length(Value) do begin
    case Value[i] of
      #$003B:begin
        if ss[Length(ss) - 1] <> '' then begin
          if ss[Length(ss) - 1][1] <> #$002E then ss[Length(ss) - 1] := #$002E + ss[Length(ss) - 1];
          SetLength(ss, Length(ss) + 1);
        end;
      end;
      #$0041..#$005A:
        ss[Length(ss) - 1] := ss[Length(ss) - 1] + WideChar(word(Value[i]) + 32);
      else ss[Length(ss) - 1] := ss[Length(ss) - 1] + Value[i];
    end;
  end;
  if ss[Length(ss) - 1] = '' then begin
    SetLength(ss, Length(ss) - 1);
  end else begin
    if ss[Length(ss) - 1][1] <> #$002E then ss[Length(ss) - 1] := #$002E + ss[Length(ss) - 1];
  end;
  if Assigned(FFileExtList) then FFileExtList.Free;
  FFileExtList := TReserveList.Create(ss, false);
end;

{ TNotePadParser }

constructor TNotePadParser.Create(target: TNotePadString;
  mode: TNotePadMode);
begin
  inherited Create;
  FNotePadMode := mode;
  FNotePadString := target;
  FSb := FNotePadString.InternalSB;
end;

procedure TNotePadParser.ParseLine(num: integer);
var
  C: WideChar;
begin
  FLine := FNotePadString[num];
  FLine.EndToken := #0;
  FLine.EndElement := #0;
  FLine.ParseDataString := '';
  FLine.PrevToken := #0;
  C := #0;
  FElement := #0;
  FParseDataString := '';
  FPrevToken := #0;
  if num > 0 then begin
    C := FNotePadString[num - 1].EndToken;
    FElement := FNotePadString[num - 1].EndElement;
    FLine.EndElement := FElement;
    FParseDataString := FNotePadString[num - 1].ParseDataString;
    FLine.ParseDataString := FParseDataString;
    FPrevToken := FNotePadString[num - 1].PrevToken;
    FLine.PrevToken := FPrevToken;
  end;
  FParsePos := FLine.Start;
  FParseEnd := FLine.Start + FLine.Length;
  FToken := #0;
  if C <> #0 then begin
    ParseToken(C);
    FPrevToken := FToken;
  end;
  while FParsePos < FParseEnd do begin
    ParseToken(true);
    //\͕fȂ낤OK
    if (FToken <> toSpace) and (FToken <> toTab) and (FLine.EndToken = #0)then
      FPrevToken := FToken;
  end;
  {if (FLine.EndToken = #0) then FLine.PrevToken := #0 else }
  FLine.PrevToken := FPrevToken;
end;

procedure TNotePadParser.ParseLineColor(num: integer);
var
  i: integer;
  C: WideChar;
  AParsePos: integer;
  ws: WideString;
begin
  FLine := FNotePadString[num];
  FLine.EndToken := #0;
  FLine.EndElement := #0;
  FLine.ParseDataString := '';
  FLine.PrevToken := #0;
  C := #0;
  FElement := #0;
  FParseDataString := '';
  FPrevToken := #0;
  if num > 0 then begin
    C := FNotePadString[num - 1].EndToken;
    FElement := FNotePadString[num - 1].EndElement;
    FLine.EndElement := FElement;
    FParseDataString := FNotePadString[num - 1].ParseDataString;
    FLine.ParseDataString := FParseDataString;
    FPrevToken := FNotePadString[num - 1].PrevToken;
    FLine.PrevToken := FPrevToken;
  end;
  FParsePos := FLine.Start;
  FParseEnd := FLine.Start + FLine.Length;
  FToken := #0;
  CheckLineLength(FLine.Length);//v|[ViΉŕK{
  if C <> #0 then begin
    AParsePos := FParsePos;
    ParseToken(C);
    for i := AParsePos to FParsePos - 1 do
      LineTokens[i - FLine.Start] := FToken;
    FPrevToken := FToken;
  end;
  while FParsePos < FParseEnd do begin
    AParsePos := FParsePos;
    ParseToken(true);
    if FToken = toAnk then begin
      ws := FSb.ToString(AParsePos, FParsePos - AParsePos);
      if FNotePadMode.FReserveList.Find(ws) then FToken := toReserve else
      if FNotePadMode.FReserve2List.Find(ws) then FToken := toReserve2 else
      if FNotePadMode.FReserve3List.Find(ws) then FToken := toReserve3;
    end;
    for i := AParsePos to FParsePos - 1 do
      LineTokens[i - FLine.Start] := FToken;
    if (FToken <> toSpace) and (FToken <> toTab) and (FLine.EndToken = #0)then
      FPrevToken := FToken;
  end;
  {if (FLine.EndToken = #0) then FLine.PrevToken := #0 else }
  FLine.PrevToken := FPrevToken;
end;

function TNotePadParser.ParseRange(start1, end1: integer): integer;
var
  i: integer;
  AEndToken, AEndElement, APrevToken: WideChar;
  AParseDataString: WideString;
begin
//
//ɂ͐܂ԂȂsȂ悤ɂ
//
  for i := start1 to end1 do ParseLine(i);
  if end1 >= FNotePadString.Count - 1 then begin
    result := end1;
    exit;
  end;
  i := end1;
  repeat
    inc(i);
    AEndToken := FNotePadString[i].EndToken;
    AEndElement := FNotePadString[i].EndElement;
    AParseDataString := FNotePadString[i].ParseDataString;
    APrevToken := FNotePadString[i].PrevToken;
    ParseLine(i);
    if i >= FNotePadString.Count - 1 then begin
      Result := i;
      exit;
    end;
  until (AEndToken = FNotePadString[i].EndToken)
    and (AEndElement = FNotePadString[i].EndElement)
    and (APrevToken = FNotePadString[i].PrevToken)
    and (AParseDataString = FNotePadString[i].ParseDataString);
  result := i;
end;

procedure TNotePadParser.ParseToken(AHttp: boolean);
begin
  if AHttp and ParseTokenHttp then exit;
  ParseToken(TokenFromFirstChar(FSb[FParsePos]));
end;

function TNotePadParser.ParseTokenHttp: boolean;
begin
  Result := false;
  case FToken of
    toUrlStart,
    toFtpStart,
    toHttpsStart,
    toRtspStart,
    toPnmStart,
    toMmsStart,
    toNewsStart,
    toNntpStart:
    begin
      ParseToken(toUrl);
      Result := true;
      exit;
    end;
  end;
  case FSb[FParsePos] of
    'f':begin
      if (FParsePos + 6 < FSb.StringLength) and
        (FSb[FParsePos + 1] = 't') and
        (FSb[FParsePos + 2] = 'p') and
        (FSb[FParsePos + 3] = ':') and
        (FSb[FParsePos + 4] = '/') and
        (FSb[FParsePos + 5] = '/') and
        IsUrlChar(FSb[FParsePos + 6])
      then begin
        ParseToken(toFtpStart);
        Result := true;
        exit;
      end;
    end;
    'h':begin
      if (FParsePos + 7 < FSb.StringLength) and
        (FSb[FParsePos + 1] = 't') and
        (FSb[FParsePos + 2] = 't') and
        (FSb[FParsePos + 3] = 'p') and
        (FSb[FParsePos + 4] = ':') and
        (FSb[FParsePos + 5] = '/') and
        (FSb[FParsePos + 6] = '/') and
        IsUrlChar(FSb[FParsePos + 7])
      then begin
        ParseToken(toUrlStart);
        Result := true;
        exit;
      end else if (FParsePos + 8 < FSb.StringLength) and
        (FSb[FParsePos + 1] = 't') and
        (FSb[FParsePos + 2] = 't') and
        (FSb[FParsePos + 3] = 'p') and
        (FSb[FParsePos + 4] = 's') and
        (FSb[FParsePos + 5] = ':') and
        (FSb[FParsePos + 6] = '/') and
        (FSb[FParsePos + 7] = '/') and
        IsUrlChar(FSb[FParsePos + 8])
      then begin
        ParseToken(toHttpsStart);
        Result := true;
        exit;
      end;
    end;
    'm':begin
      if (FParsePos + 6 < FSb.StringLength) and
        (FSb[FParsePos + 1] = 'm') and
        (FSb[FParsePos + 2] = 's') and
        (FSb[FParsePos + 3] = ':') and
        (FSb[FParsePos + 4] = '/') and
        (FSb[FParsePos + 5] = '/') and
        IsUrlChar(FSb[FParsePos + 6])
      then begin
        ParseToken(toMmsStart);
        Result := true;
        exit;
      end;
    end;
    'n':begin
      if (FParsePos + 7 < FSb.StringLength) and
        (FSb[FParsePos + 1] = 'e') and
        (FSb[FParsePos + 2] = 'w') and
        (FSb[FParsePos + 3] = 's') and
        (FSb[FParsePos + 4] = ':') and
        (FSb[FParsePos + 5] = '/') and
        (FSb[FParsePos + 6] = '/') and
        IsUrlChar(FSb[FParsePos + 7])
      then begin
        ParseToken(toNewsStart);
        Result := true;
        exit;
      end else if (FParsePos + 7 < FSb.StringLength) and
        (FSb[FParsePos + 1] = 'n') and
        (FSb[FParsePos + 2] = 't') and
        (FSb[FParsePos + 3] = 'p') and
        (FSb[FParsePos + 4] = ':') and
        (FSb[FParsePos + 5] = '/') and
        (FSb[FParsePos + 6] = '/') and
        IsUrlChar(FSb[FParsePos + 7])
      then begin
        ParseToken(toNntpStart);
        Result := true;
        exit;
      end;
    end;
    'p':begin
      if (FParsePos + 6 < FSb.StringLength) and
        (FSb[FParsePos + 1] = 'n') and
        (FSb[FParsePos + 2] = 'm') and
        (FSb[FParsePos + 3] = ':') and
        (FSb[FParsePos + 4] = '/') and
        (FSb[FParsePos + 5] = '/') and
        IsUrlChar(FSb[FParsePos + 6])
      then begin
        ParseToken(toPnmStart);
        Result := true;
        exit;
      end;
    end;
    'r':begin
      if (FParsePos + 7 < FSb.StringLength) and
        (FSb[FParsePos + 1] = 't') and
        (FSb[FParsePos + 2] = 's') and
        (FSb[FParsePos + 3] = 'p') and
        (FSb[FParsePos + 4] = ':') and
        (FSb[FParsePos + 5] = '/') and
        (FSb[FParsePos + 6] = '/') and
        IsUrlChar(FSb[FParsePos + 7])
      then begin
        ParseToken(toRtspStart);
        Result := true;
        exit;
      end;
    end;
  end;
end;

procedure TNotePadParser.ParseToken(const C: WideChar);
begin
  FToken := C;
  case C of
    toSBSymbol:SymbolProc;
    toSBInt: IntegerProc;
    toAnk: AnkProc;
    toAnkNoReserve: Ank2Proc;
    toDBSymbol: DBSymbolProc;
    toDBInt: DBIntProc;
    toDBAlph: DBAlphProc;
    toDBHira: DBHiraProc;
    toDBKana: DBKanaProc;
    toDBKanji: DBKanjiProc;
    toSBKana: SBKanaProc;
    toSBKanaSymbol: SBKanaSymbolProc;
    toUrlStart: UrlStartProc;
    toFtpStart: FtpStartProc;
    toHttpsStart: HttpsStartProc;
    toRtspStart: RtspStartProc;
    toPnmStart: PnmStartProc;
    toMmsStart: MmsStartProc;
    toNewsStart: NewsStartProc;
    toNntpStart: NntpStartProc;
    toUrl: UrlProc;
    else NormalProc;
  end;
end;

function TNotePadParser.IsFirstCharOfLine: boolean;
//s@ɕ\邩H
//܂Ԃč[ɗꍇ͍sł͂Ȃ
//āAs`FbN̂тɊSȂ̂ڂ肪KvɂȂB
var
  i: integer;
begin
  Result := false;
//  if FParsePos > FLine.Start then exit;
//  if (FLine.Start > 0) and (FSb[FLine.Start - 1] <> #13) and (FSb[FLine.Start - 1] <> #10) then exit;
//  for i := FLine.Start to FParsePos - 1 do if (FSb[i] <> ' ') or (FSb[i] <> #9)  then exit;
  for i := FParsePos - 1 downto 0 do begin
    case FSb[i] of
      #10: break;
      #9, ' ':begin end;
      else exit;
    end;
  end;
  Result := true;
end;

function TNotePadParser.IsVeryFirstCharOfLine: boolean;
begin
  if (FParsePos > 0) and (FSb[FParsePos - 1] <> #10) then
    Result := false else Result := true;
end;

procedure TNotePadParser.NormalProc;
begin
  inc(FParsePos);
end;

procedure TNotePadParser.SymbolProc;
begin
  inc(FParsePos);
end;

procedure TNotePadParser.IntegerProc;
var
  C: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C := FSb[FParsePos];
  until (C < WideChar('0')) or (C > WideChar('9'));
end;

procedure TNotePadParser.AnkProc;
var
  C: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C := FSb[FParsePos];
  until (C < #$0030) or ((C <= #$0040) and (#$003A <= C)) or
    ((C <= #$005E) and (C >= #$005B)) or (C = #$0060) or (C > #$007A);
end;

procedure TNotePadParser.Ank2Proc;
var
  C: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C := FSb[FParsePos];
  until (C <= #$0040)  or
    ((C <= #$005E) and (C >= #$005B)) or (C = #$0060) or (C > #$007A);
end;

procedure TNotePadParser.Ank3Proc;
var
  C: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C := FSb[FParsePos];
  until ((C < #$0030) and (C <> '-')) or ((C <= #$0040) and (#$003A <= C)) or
    ((C <= #$005E) and (C >= #$005B)) or (C = #$0060) or (C > #$007A);
end;

procedure TNotePadParser.DBSymbolProc;
begin
  inc(FParsePos);
end;

procedure TNotePadParser.DBIntProc;
var
  C: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C := FSb[FParsePos];
  until (C < #$FF10) or (C > #$FF19);
end;

procedure TNotePadParser.DBAlphProc;
var
  C: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C := FSb[FParsePos];
  until (C < #$FF21) or ((C > #$FF5A) or ((C > #$FF3A) and (C < #$FF41)));
end;

procedure TNotePadParser.DBHiraProc;
var
  C: WideChar;
  b: boolean;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C := FSb[FParsePos];
    case TokenFromFirstChar(C) of
      toDBHira: b := true;
      else b := (C = #$30FC);
    end;
    {case GetUnicodeSubset(C) of
      usHiragana: b := true;
      else begin
        b := (C = #$30FC);
      end;
    end;}
  until not b;
end;

procedure TNotePadParser.DBKanaProc;
var
  C: WideChar;
  b: boolean;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C := FSb[FParsePos];
    case TokenFromFirstChar(C) of
      toDBKana: b := true;
      else b := (#$3099 <= C) and (C <= #$309C);
    end;
    {case GetUnicodeSubset(C) of
      usKatakana: b := true;
      else begin
        b := (#$3099 <= C) and (C <= #$309C);
      end;
    end;   }
  until not b;
end;

procedure TNotePadParser.DBKanjiProc;
label
  Loop1;
begin
  Loop1:
  if FParsePos >= FParseEnd then begin
    FLine.EndToken := FToken;
    exit;
  end;
  inc(FParsePos);
  if FParsePos >= FSb.StringLength then exit;
  case TokenFromFirstChar(FSb[FParsePos]) of
    toDBKanji: goto Loop1;
  end;
  {case GetUnicodeSubset(FSb[FParsePos]) of
    usCJKIdeographs,
    usCJKCompatibilityIdeographs: goto Loop1;
    usCJKSymbolsPunctuation: begin
      case FSb[FParsePos] of
        #$3003: goto Loop1;//'V'
        #$3005..#$3007: goto Loop1;//'X''Y''Z'
      end;
    end;
  end;}
end;

procedure TNotePadParser.SBKanaProc;
var
  C: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C := FSb[FParsePos];
  until (C < #$FF66) or (C > #$FF9F);
end;

procedure TNotePadParser.SBKanaSymbolProc;
begin
  inc(FParsePos);
end;

procedure TNotePadParser.FtpStartProc;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until (FParsePos >= 6) and (
    (FSb[FParsePos - 6] = WideChar('f')) and (FSb[FParsePos - 5] = WideChar('t')) and
    (FSb[FParsePos - 4] = WideChar('p')) and (FSb[FParsePos - 3] = WideChar(':')) and
    (FSb[FParsePos - 2] = WideChar('/')) and (FSb[FParsePos - 1] = WideChar('/'))
  );
  if FParsePos >= FParseEnd then FLine.EndToken := toUrl;
end;

procedure TNotePadParser.HttpsStartProc;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until (FParsePos >= 8) and (
    (FSb[FParsePos - 8] = WideChar('h')) and (FSb[FParsePos - 7] = WideChar('t')) and
    (FSb[FParsePos - 6] = WideChar('t')) and (FSb[FParsePos - 5] = WideChar('p')) and
    (FSb[FParsePos - 4] = WideChar('s')) and (FSb[FParsePos - 3] = WideChar(':')) and
    (FSb[FParsePos - 2] = WideChar('/')) and (FSb[FParsePos - 1] = WideChar('/'))
  );
  if FParsePos >= FParseEnd then FLine.EndToken := toUrl;
end;

procedure TNotePadParser.MmsStartProc;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until (FParsePos >= 6) and (
    (FSb[FParsePos - 6] = WideChar('m')) and (FSb[FParsePos - 5] = WideChar('m')) and
    (FSb[FParsePos - 4] = WideChar('s')) and (FSb[FParsePos - 3] = WideChar(':')) and
    (FSb[FParsePos - 2] = WideChar('/')) and (FSb[FParsePos - 1] = WideChar('/'))
  );
  if FParsePos >= FParseEnd then FLine.EndToken := toUrl;
end;

procedure TNotePadParser.NewsStartProc;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until (FParsePos >= 7) and (               (FSb[FParsePos - 7] = WideChar('n')) and
    (FSb[FParsePos - 6] = WideChar('e')) and (FSb[FParsePos - 5] = WideChar('w')) and
    (FSb[FParsePos - 4] = WideChar('s')) and (FSb[FParsePos - 3] = WideChar(':')) and
    (FSb[FParsePos - 2] = WideChar('/')) and (FSb[FParsePos - 1] = WideChar('/'))
  );
  if FParsePos >= FParseEnd then FLine.EndToken := toUrl;
end;

procedure TNotePadParser.NntpStartProc;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until (FParsePos >= 7) and (               (FSb[FParsePos - 7] = WideChar('n')) and
    (FSb[FParsePos - 6] = WideChar('n')) and (FSb[FParsePos - 5] = WideChar('t')) and
    (FSb[FParsePos - 4] = WideChar('p')) and (FSb[FParsePos - 3] = WideChar(':')) and
    (FSb[FParsePos - 2] = WideChar('/')) and (FSb[FParsePos - 1] = WideChar('/'))
  );
  if FParsePos >= FParseEnd then FLine.EndToken := toUrl;
end;

procedure TNotePadParser.PnmStartProc;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until (FParsePos >= 6) and (
    (FSb[FParsePos - 6] = WideChar('p')) and (FSb[FParsePos - 5] = WideChar('n')) and
    (FSb[FParsePos - 4] = WideChar('m')) and (FSb[FParsePos - 3] = WideChar(':')) and
    (FSb[FParsePos - 2] = WideChar('/')) and (FSb[FParsePos - 1] = WideChar('/'))
  );
  if FParsePos >= FParseEnd then FLine.EndToken := toUrl;
end;

procedure TNotePadParser.RtspStartProc;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until (FParsePos >= 7) and (               (FSb[FParsePos - 7] = WideChar('r')) and
    (FSb[FParsePos - 6] = WideChar('t')) and (FSb[FParsePos - 5] = WideChar('s')) and
    (FSb[FParsePos - 4] = WideChar('p')) and (FSb[FParsePos - 3] = WideChar(':')) and
    (FSb[FParsePos - 2] = WideChar('/')) and (FSb[FParsePos - 1] = WideChar('/'))
  );
  if FParsePos >= FParseEnd then FLine.EndToken := toUrl;
end;

procedure TNotePadParser.UrlStartProc;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until (FParsePos >= 7) and (               (FSb[FParsePos - 7] = WideChar('h')) and
    (FSb[FParsePos - 6] = WideChar('t')) and (FSb[FParsePos - 5] = WideChar('t')) and
    (FSb[FParsePos - 4] = WideChar('p')) and (FSb[FParsePos - 3] = WideChar(':')) and
    (FSb[FParsePos - 2] = WideChar('/')) and (FSb[FParsePos - 1] = WideChar('/'))
  );
  if FParsePos >= FParseEnd then FLine.EndToken := toUrl;
end;

procedure TNotePadParser.UrlProc;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until (not IsUrlChar(FSb[FParsePos]));
end;

procedure TNotePadParser.SingleQuotationProc;
var
  C1, C2: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    C2 := FSb[FParsePos];
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C1 := FSb[FParsePos];
  until (C2 = #$27) or (C1 = #10);
end;

procedure TNotePadParser.DoubleQuotationProc;
var
  C1, C2: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    C2 := FSb[FParsePos];
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C1 := FSb[FParsePos];
  until (C2 = #$22) or (C1 = #10);
end;

procedure TNotePadParser.CommentBrock0Proc;
var
  C2: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    C2 := FSb[FParsePos];
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until (C2 = #$7D);
end;

procedure TNotePadParser.CommentProc;
var
  C1: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C1 := FSb[FParsePos];
  until (C1 = #10);
end;

procedure TNotePadParser.CommentBrock2StartProc;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until (FSb[FParsePos - 2] =#$28 ) and (FSb[FParsePos - 1] =#$2A);
  if FParsePos >= FParseEnd then FLine.EndToken := toCommentBrock2;
end;

procedure TNotePadParser.CommentBrock2Proc(splitHit: integer);
var
  C2, C3: WideChar;
  b: boolean;
  AStart: integer;
begin
  C2 := #0;
  AStart := FParsePos - splitHit;
  repeat
    if FParsePos >= FParseEnd then begin
      if C2 = #$2A then
        FLine.EndToken := toCommentBrock2sp
      else
        FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= Fsb.StringLength then exit;
    C3 := FSb[FParsePos - 2];
    C2 := FSb[FParsePos - 1];
    b := (C3 = #$2A) and (C2 = #$29) and (AStart + 1 <> FParsePos);
  until b;
end;

procedure TNotePadParser.HexProc;
var
  b: boolean;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    case FSb[FParsePos] of
      #$30..#$39, #$41..#$46, #$61..#$66: b := false;
      else b := true;
    end;
  until b;
end;

procedure TNotePadParser.TagStartProc;
begin
  if FParsePos >= FParseEnd then
    FLine.EndToken := FToken
  else begin
    inc(FParsePos);
    FElement := elementTag;
    FLine.EndElement := FElement;
  end;
end;

procedure TNotePadParser.TagEndProc;
begin
  if FParsePos >= FParseEnd then
    FLine.EndToken := FToken
  else begin
    inc(FParsePos);
    FElement := #0;
    FLine.EndElement := FElement;
  end;
end;

procedure TNotePadParser.CommentBrock3Proc(splitHit: integer);
var
  C1, C2, C3, C4: WideChar;
  b: boolean;
  AStart: integer;
begin
  C1 := #0;
  C2 := #0;
  C3 := #0;
  AStart := FParsePos - splitHit;
  repeat
    if FParsePos >= FParseEnd then begin
      if ((C2 = #$2D) and (C1 = #$2D) and (FParsePos + 1 < FSb.StringLength) and (FSb[FParsePos + 1] = #$3E)) or
         ((C3 = #$2D) and (C2 = #$2D) and (C1 = #$3E)) then
        FLine.EndToken := toCommentBrock3sp
      else
        FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= Fsb.StringLength then exit;
    C3 := FSb[FParsePos - 2];
    C2 := FSb[FParsePos - 1];
    C1 := FSb[FParsePos];
    if FParsePos < 3 then C4 := #0 else C4 := FSb[FParsePos - 3];
    b := (C4 = #$2D) and (C3 = #$2D) and (C2 = #$3E) and (AStart + 1 < FParsePos);
  until b;
end;

procedure TNotePadParser.CommentBrock3StartProc;
begin
//<!--ƁAlłĂȂƂɂ͗ȂA
//܂Ԃ4O܂ŊmFłȂȂB
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := toCommentBrock3Start;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until (FSb[FParsePos - 4] = #$3C ) and (FSb[FParsePos - 3] = #$21) and
        (FSb[FParsePos - 2] = #$2D ) and (FSb[FParsePos - 1] = #$2D);
  if FParsePos >= FParseEnd then FLine.EndToken := toCommentBrock3;
end;

procedure TNotePadParser.DoubleQuotSkipProc(skip: boolean);
var
  C1, C2: WideChar;
  b: boolean;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      if skip then FLine.EndToken := toDoubleQuotSkip else FLine.EndToken := FToken;
      exit;
    end;
    C2 := FSb[FParsePos];
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C1 := FSb[FParsePos];
    if skip then begin
      skip := false;
      b := false;
    end else begin
      skip := (C2 = #$5C) and (C1 <> #10);
      b := (not skip) and ((C1 = #10) or (C2 = #$22));
    end;
  until b;
end;

procedure TNotePadParser.SingleQuotSkipProc(skip: boolean);
var
  C1, C2: WideChar;
  b: boolean;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      if skip then FLine.EndToken := toSingleQuotSkip else FLine.EndToken := FToken;
      exit;
    end;
    C2 := FSb[FParsePos];
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C1 := FSb[FParsePos];
    if skip then begin
      skip := false;
      b := false;
    end else begin
      skip := (C2 = #$5C) and (C1 <> #10);
      b := (not skip) and ((C1 = #10) or (C2 = #$27));
    end;
  until b;
end;

procedure TNotePadParser.AtDoubleQuotStartProc;
var
  C2, C3: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := toAtDoubleQuotStart;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= Fsb.StringLength then exit;
    C3 := FSb[FParsePos - 2];
    C2 := FSb[FParsePos - 1];
  until (C3 = #$40) and (C2 = #$22);
  if FParsePos >= FParseEnd then FLine.EndToken := toAtDoubleQuot;
end;

procedure TNotePadParser.AtDoubleQuotProc;
var
  C2: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    C2 := FSb[FParsePos];
    inc(FParsePos);
    if FParsePos >= Fsb.StringLength then exit;
  until C2 = #$22;
end;

procedure TNotePadParser.CommentBrock1StartProc;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := toCommentBrock1Start;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until (FSb[FParsePos - 2] = '/') and (FSb[FParsePos - 1] = '*');
  if FParsePos >= FParseEnd then FLine.EndToken := toCommentBrock1;
end;

procedure TNotePadParser.CommentBrock1Proc(splitHit: integer);
var
  C2, C3: WideChar;
  b: boolean;
  AStart: integer;
begin
  C2 := #0;
  AStart := FParsePos - splitHit;
  repeat
    if FParsePos >= FParseEnd then begin
      if C2 = '*' then
        FLine.EndToken := toCommentBrock1sp
      else
        FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= Fsb.StringLength then exit;
    C3 := FSb[FParsePos - 2];
    C2 := FSb[FParsePos - 1];
    b := (C3 = '*') and (C2 = '/') and (AStart + 1 <> FParsePos);
  until b;
end;

procedure TNotePadParser.TagQuestStartProc;
begin
  if FParsePos >= FParseEnd then
    FLine.EndToken := FToken
  else begin
    inc(FParsePos);
    FElement := elementTagQuest;
    FLine.EndElement := FElement;
  end;
end;

procedure TNotePadParser.TagQuestEndProc;
begin
  if FParsePos >= FParseEnd then
    FLine.EndToken := FToken
  else begin
    inc(FParsePos);
    FElement := #0;
    FLine.EndElement := FElement;
  end;
end;

procedure TNotePadParser.PerlHereEndProc;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until FParseDataString = FSb.ToString(FParsePos -
    Length(FParseDataString), Length(FParseDataString));
  FParseDataString := '';
  FLine.ParseDataString := '';
  FElement := #0;
  FLine.EndElement := #0;
end;

procedure TNotePadParser.PerlHereStartProc;
var
  C2, C3: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= Fsb.StringLength then exit;
    C3 := FSb[FParsePos - 2];
    C2 := FSb[FParsePos - 1];
  until (C3 = '<') and (C2 = '<');
  if FParsePos >= FParseEnd then FLine.EndToken := toHereStartWord;
end;

procedure TNotePadParser.PerlHereStartWordProc;
var
  C1: WideChar;
  AStart, AEnd: integer;
begin
  //ڍׂPhpHereStartWordProc
  AStart := FParsePos;
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C1 := FSb[FParsePos];
  until (C1 = #10) or (C1 = ';');
  while (AStart >= 0) and (FSb[AStart] <> '<') do dec(AStart);
  inc(AStart);
  //while GetUnicodeCategory(FSb[AStart]) = ucSpaceSeparator do inc(AStart);
  while FSb[AStart] = #$20 do inc(AStart);//SpXy[XƂ͈Ⴄ񂶂Ȃ
  if (FSb[AStart] = '"') or (FSb[AStart] = '''') or (FSb[AStart] = '`') then inc(AStart);
  AEnd := FParsePos;
  while (AEnd - 1 >= AStart) and
    //(GetUnicodeCategory(FSb[AEnd - 1]) = ucSpaceSeparator) do dec(AEnd);
    (FSb[AEnd - 1] = #$20) do dec(AEnd);
  if (AEnd >= AStart) and
     ((FSb[AEnd - 1] = '"') or (FSb[AEnd - 1] = '''') or (FSb[AEnd - 1] = '`'))
    then dec(AEnd);
  if AEnd {- 1} > AStart then begin
    FParseDataString := FSb.ToString(AStart, AEnd - AStart);
  end else begin
    FParseDataString := '';
  end;
  FLine.ParseDataString := FParseDataString;
end;

procedure TNotePadParser.RegExpSkipProc(skip: boolean);
var
  C1, C2: WideChar;
  b: boolean;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      if skip then FLine.EndToken := toRegExpSkip else FLine.EndToken := FToken;
      exit;
    end;
    C2 := FSb[FParsePos];
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C1 := FSb[FParsePos];
    if skip then begin
      skip := false;
      b := false;
    end else begin
      skip := (C2 = '\') and (C1 <> #10);
      b := (not skip) and ((C1 = #10) or (C2 = '/'));
    end;
  until b;
end;

procedure TNotePadParser.MatchStartProc;
var
  C2, C3: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := toMatchStart;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= Fsb.StringLength then exit;
    C3 := FSb[FParsePos - 2];
    C2 := FSb[FParsePos - 1];
  until (C3 = 'm') and (C2 = '/');
  if FParsePos >= FParseEnd then FLine.EndToken := toRegExp;
end;

procedure TNotePadParser.SubStartProc;
var
  C2, C3: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := toSubStart;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= Fsb.StringLength then exit;
    C3 := FSb[FParsePos - 2];
    C2 := FSb[FParsePos - 1];
  until (C3 = 's') and (C2 = '/');
  if FParsePos >= FParseEnd then FLine.EndToken := toSubMatch;
end;

procedure TNotePadParser.SubMatchSkipProc(skip: boolean);
var
  C1, C2: WideChar;
  b: boolean;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      if skip then FLine.EndToken := toSubMatchSkip else FLine.EndToken := FToken;
      exit;
    end;
    C2 := FSb[FParsePos];
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C1 := FSb[FParsePos];
    if skip then begin
      skip := false;
      b := false;
    end else begin
      skip := (C2 = '\') and (C1 <> #10);
      b := (not skip) and ((C1 = #10) or (C2 = '/'));
    end;
  until b;
end;

procedure TNotePadParser.DoubleQuotationHtmlProc;
var
  C2: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    C2 := FSb[FParsePos];
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until C2 = #$22;
end;

procedure TNotePadParser.SingleQuotationHtmlProc;
var
  C2: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    C2 := FSb[FParsePos];
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until C2 = #$27;
end;

procedure TNotePadParser.PhpHereStartProc;
var
  C2, C3, C4: WideChar;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= Fsb.StringLength then exit;
    C4 := FSb[FParsePos - 3];
    C3 := FSb[FParsePos - 2];
    C2 := FSb[FParsePos - 1];
  until (C4 = '<') and (C3 = '<') and (C2 = '<');
  if FParsePos >= FParseEnd then FLine.EndToken := toHereStartWord;
end;

procedure TNotePadParser.PhpHereStartWordProc;
var
  C1: WideChar;
  AStart, AEnd: integer;
begin
  //킩ȂȂ!
  //rexit̎̓qAF̂ǂB
  //qAFAEnd-1gĂ̂AStart-1gĂȂ̂͂ȂŁH
  AStart := FParsePos;
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C1 := FSb[FParsePos];
  until (C1 = #10) or (C1 = ';');
  while (AStart >= 0) and (FSb[AStart] <> '<') do dec(AStart);
  inc(AStart);
  //while GetUnicodeCategory(FSb[AStart]) = ucSpaceSeparator do inc(AStart);
  while FSb[AStart] = #$20 do inc(AStart);//SpXy[XƂ͈Ⴄ񂶂Ȃ
  if (FSb[AStart] = '"') or (FSb[AStart] = '''') or (FSb[AStart] = '`') then inc(AStart);
  AEnd := FParsePos;
  while (AEnd - 1 >= AStart) and
    //(GetUnicodeCategory(FSb[AEnd - 1]) = ucSpaceSeparator) do dec(AEnd);
    (FSb[AEnd - 1] = #$20) do dec(AEnd);
  if (AEnd >= AStart) and
     ((FSb[AEnd - 1] = '"') or (FSb[AEnd - 1] = '''') or (FSb[AEnd - 1] = '`'))
    then dec(AEnd);
  if AEnd {- 1} > AStart then begin //RR-11qA󂯕tȂ
                                    //
    FParseDataString := FSb.ToString(AStart, AEnd - AStart);
  end else begin
    FParseDataString := '';
  end;
  FLine.ParseDataString := FParseDataString;
  FElement := #0;
  FLine.EndElement := #0;
end;

procedure TNotePadParser.PhpHereEndProc;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      FLine.EndToken := FToken;
      exit;
    end;
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
  until FParseDataString = FSb.ToString(FParsePos -
    Length(FParseDataString), Length(FParseDataString));
  FParseDataString := '';
  FLine.ParseDataString := '';
  FElement := elementTagQuest;
  FLine.EndElement := elementTagQuest;
end;

procedure TNotePadParser.PhpDoubleQuotSkipProc(skip: boolean);
var
  C1, C2: WideChar;
  b: boolean;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      if skip then FLine.EndToken := toDoubleQuotSkip else FLine.EndToken := FToken;
      exit;
    end;
    C2 := FSb[FParsePos];
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C1 := FSb[FParsePos];
    if skip then begin
      skip := false;
      b := false;
    end else begin
      skip := (C2 = #$5C) and (C1 <> #10);
      b := (not skip) and (C2 = #$22);
    end;
  until b;
end;

procedure TNotePadParser.PhpSingleQuotSkipProc(skip: boolean);
var
  C1, C2: WideChar;
  b: boolean;
begin
  repeat
    if FParsePos >= FParseEnd then begin
      if skip then FLine.EndToken := toSingleQuotSkip else FLine.EndToken := FToken;
      exit;
    end;
    C2 := FSb[FParsePos];
    inc(FParsePos);
    if FParsePos >= FSb.StringLength then exit;
    C1 := FSb[FParsePos];
    if skip then begin
      skip := false;
      b := false;
    end else begin
      skip := (C2 = #$5C) and (C1 <> #10);
      b := (not skip) and (C2 = #$27);
    end;
  until b;
end;

procedure TNotePadParser.PodCommentProc;
  label loop1;
begin
loop1:
  if FParsePos >= FParseEnd then begin
    FLine.EndToken := FToken;
    exit;
  end;
  inc(FParsePos);
  if FParsePos >= FSb.StringLength then exit;
  goto loop1;
end;

end.
