(*********************************************************************

  TKeyMacro ver 1.3

  start  2001/04/30
  update 2001/06/28

  Copyright (c) 2001 {cF <katsuhiko.honda@nifty.ne.jp>

  --------------------------------------------------------------------
  L[{[h}NR|[lg

**********************************************************************)

unit HKeyMacro;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, Dialogs;

type
  PKeyMacroData = ^TKeyMacroData;
  TKeyMacroData = record
    Msg: Cardinal;
    CharCode: Word;
    KeyData: Longint;
    Shift, Ctrl, Alt: Byte;
  end;

  TKeyMacroStatus = (msNone, msRecording, msWaiting, msRunning);

  TKeyMacro = class;

  TKeyMacroDataList = class(TList)
  protected
    FKeyMacro: TKeyMacro;
  public
    destructor Destroy; override;
    procedure Clear; {$IFNDEF VER100}{$IFNDEF VER90} override; {$ENDIF}{$ENDIF}
    function Add(Message: TMessage): PKeyMacroData; virtual;
  end;

  TKeyMacro = class(TComponent)
  private
    FControl: TWinControl;            // bZ[W擾ΏۃRg[
    FIMECharCount: Integer;           // WM_IME_CHAR bZ[WJE^
    FList: TKeyMacroDataList;         // bZ[Wf[^Xg
    FOriginalWndProcPoint: Pointer;   // FControl  WndProc ւ̃|C^
    FRecordingWndProcPoint: Pointer;  // RecordingWndProc ւ̃|C^
    FShowDialog: Boolean;             // ShowMessage 邵ȂtO
    FStatus: TKeyMacroStatus;         // ԃtO
    FOnChange: TNotifyEvent;          // ԑJڃCxg
  protected
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure SetControl(Value: TWinControl); virtual;
    procedure SetRecordingWndProc; virtual;    // TuNX
    procedure RestoreOriginalWndProc; virtual; // TuNX
    procedure RecordingWndProc(var Message: TMessage); virtual;
    procedure SetStatus(Value: TKeyMacroStatus); virtual;
    procedure DoChange; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure BeginRec; virtual;                // L^Jn
    procedure EndRec; virtual;                  // L^I
    procedure CancelRec; virtual;               // L^f
    procedure Execute; virtual;                 // s
    procedure CancelExe; virtual;               // sf
    procedure Clear; virtual;
    procedure MacroDataToStream(Stream: TStream); virtual;
    function StreamToMacroData(Stream: TStream): Boolean; virtual;
    procedure LoadFromFile(const FileName: string); virtual;
    procedure LoadFromStream(Stream: TStream); virtual;
    procedure SaveToFile(const FileName: string); virtual;
    procedure SaveToStream(Stream: TStream); virtual;
    property Status: TKeyMacroStatus read FStatus write SetStatus;
  published
    property Control: TWinControl read FControl write SetControl;
    property ShowDialog: Boolean read FShowDialog write FShowDialog;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('katsuhiko', [TKeyMacro]);
end;


{ TKeyMacroDataList }

destructor TKeyMacroDataList.Destroy;
begin
  Clear;
  inherited Destroy;
end;

procedure TKeyMacroDataList.Clear;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    Dispose(PKeyMacroData(Items[I])); // ێĂ郁
  inherited Clear;                    // TList  Clear Ă
end;

function TKeyMacroDataList.Add(Message: TMessage): PKeyMacroData;
var
  Buf: PKeyMacroData;
begin
  Result := nil;

  // }Nf[^[hƂ́AMessage.Msg = 0 ŎsdlƂ
  if Message.Msg = 0 then
  begin
    New(Buf);
    inherited Add(Buf);
    Result := Buf;
    Exit;
  end;

  // TMemo ł́AWM_IME_CHAR őĂioCgł͂Ȃj
  //  WM_CHAR KĂ̂ŁALZB
  if Message.Msg = WM_IME_CHAR then
//    Inc(FKeyMacro.FIMECharCount)
  else
    if (Message.Msg = CN_CHAR) and (FKeyMacro.FIMECharCount > 0) then
    begin
      Dec(FKeyMacro.FIMECharCount);
      Exit;
    end;
  // CharCode ́APoCgAQoCgA
  // ׂ Word(Message.WParam)Ŏ擾oi悤j
  New(Buf);
  Buf.Msg := Message.Msg;
  Buf.CharCode := Word(Message.WParam);
  Buf.KeyData := Message.LParam;
  Buf.Shift := Byte(GetKeyState(VK_SHIFT));
  Buf.Ctrl := Byte(GetKeyState(VK_CONTROL));
  Buf.Alt := Byte(GetKeyState(VK_MENU));
  inherited Add(Buf);
  Result := Buf;
end;


{ TKeyMacto }

constructor TKeyMacro.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  // L^p WndProc ւ̃|C^擾
  FRecordingWndProcPoint :=  Classes.MakeObjectInstance(RecordingWndProc);
  FList := TKeyMacroDataList.Create;
  FList.FKeyMacro := Self;
end;

destructor TKeyMacro.Destroy;
begin
  // L^ȂΏI
  if FStatus = msRecording then
    EndRec;
  // sȂΏI
  if FStatus = msRunning then
    CancelExe;
  // L^p|C^j
  Classes.FreeObjectInstance(FRecordingWndProcPoint);
  // bZ[WXgj
  FList.Free;
  inherited Destroy;
end;

procedure TKeyMacro.Clear;
begin
  case FStatus of
    msNone:
      ; // no operation
    msRecording:
      EndRec;
    msWaiting:
      ; // no operation
    msRunning:
      CancelExe;
  end;
  Status := msNone;
  FList.Clear;
end;

procedure TKeyMacro.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = FControl) and (Operation = opRemove) then
    FControl := nil;
end;

procedure TKeyMacro.SetRecordingWndProc;
// ΏۃRg[TuNX
begin
  if (FControl <> nil) and (FRecordingWndProcPoint <> nil) then
  begin
    // ΏۃRg[ WndProc ۑ
    FOriginalWndProcPoint :=
      Pointer(GetWindowLongW(FControl.Handle, GWL_WNDPROC));
    // TuNX
    SetWindowLongW(
      FControl.Handle, GWL_WNDPROC, Longint(FRecordingWndProcPoint));
  end;
end;

procedure TKeyMacro.RestoreOriginalWndProc;
// ΏۃRg[ WndProc 𕜋A
begin
  if (FControl <> nil) and (FOriginalWndProcPoint <> nil) then
    SetWindowLongW(
      FControl.Handle, GWL_WNDPROC, Longint(FOriginalWndProcPoint));
end;

procedure TKeyMacro.SetControl(Value: TWinControl);
// ΏۃRg[̎擾
begin
  if FControl <> Value then
    if FStatus <> msRecording then
      FControl := Value
    else
    begin
      // bZ[WL^͌ɖ߂
      RestoreOriginalWndProc;
      // VRg[擾
      FControl := Value;
      if FControl <> nil then
        // nil łȂ΃TuNX
        SetRecordingWndProc
      else
        // nil Ȃ΁AL^~
        CancelRec;
    end;
end;

procedure TKeyMacro.SetStatus(Value: TKeyMacroStatus);
// ԑJڂ OnChange CxgĂяo
begin
  FStatus := Value;
  if not (csDestroying in ComponentState) then
    DoChange;
end;

procedure TKeyMacro.DoChange;
// OnChange CxgnhĂяo
begin
  if Assigned(FOnChange) then
    FOnChange(Self);
end;

procedure TKeyMacro.BeginRec;
// L^Jn
begin
  if (FControl <> nil) and
     not (FStatus in [msRecording, msRunning]) then
  begin
    FList.Clear;
    Status := msRecording;
    SetRecordingWndProc;
  end;
end;

procedure TKeyMacro.EndRec;
// L^I
begin
  if FStatus = msRecording then
  begin
    Status := msWaiting;
    RestoreOriginalWndProc;
  end;
end;

procedure TKeyMacro.CancelRec;
// L^~
begin
  if FStatus = msRecording then
  begin
    EndRec;
    FList.Clear;
    Status := msNone;
  end;
end;

procedure TKeyMacro.CancelExe;
// s~
begin
  if FStatus = msRunning then
    Status := msWaiting;
end;

procedure TKeyMacro.RecordingWndProc(var Message: TMessage);
// bZ[WL^p WndProc
begin
  case Message.Msg of
    WM_IME_CHAR,
    CN_KEYDOWN, CN_CHAR:
      FList.Add(Message); // ۑ
  end;
  // ΏۃRg[{ WndProc Ăяo
  Message.Result :=
    CallWindowProcW(FOriginalWndProcPoint,
      FControl.Handle, Message.Msg, Message.WParam, Message.LParam);
end;

procedure TKeyMacro.Execute;
// s cf keymacrodev.txt
var
  I: Integer;
  KeyState, OldKeyState: TKeyboardState;
  M: TMsg;
begin
  if (FStatus <> msWaiting) or (FControl = nil) or (FList.Count = 0) then
    Exit;
  // ̗\f[^Ƃĕۑ
  GetKeyboardState(OldKeyState);
  try
    Status := msRunning;
    for I := 0 to FList.Count - 1 do
      with PKeyMacroData(FList.Items[I])^ do
      begin
        (*
          j[̃V[gJbgɂă}NsA
          [v̓rŃL[{[h𗣂ꍇAXs[h
          {^ŋL^}Nj[̃V[gJbgŎs
          ꍇ́AKeyState ̐Ă܂̂ŁA
          Xg̊eڂ̃}Nf[^PssxAO
          L[{[h̏ԂۑEA (*1)
        *)
        GetKeyboardState(OldKeyState); // (*1)
        // ssShift, ssCtrl, ssAlt ̏ԂČ
        GetKeyboardState(KeyState);
        KeyState[VK_SHIFT] := Shift;
        KeyState[VK_CONTROL] := Ctrl;
        KeyState[VK_MENU] := Alt;
        SetKeyboardState(KeyState);

        // WM_IME_CHAR
        if Msg = WM_IME_CHAR then
        begin
          (*
              WM_IME_CHAR  Windows  DefWindowProc ɂāAQ
            P WM_CHAR ɕϊAbZ[WL[ɗ܂B
            AvP[VɂɂȂ܂ŏȂ̂ŁAPڂo
            āADispatch B
              Windows WRg[ TMemo  TEdit  WM_CHAR bZ
            [Wnhł́AWParam  LeadBytes ̏ꍇAbZ[WL[
            Qڂ WM_CHAR oAQoCgƂĈĂ
            Bi悤j
              Application.ProcessMessages ł́AL[ɂȂ܂Ń[v
            ̂ŁAł̏ɂ͑ȂB
          *)
          SendMessageW(FControl.Handle, Msg, CharCode, KeyData);
          if (PeekMessageW(M, 0, 0, 0, PM_REMOVE)) and (M.Message <> WM_QUIT) then
          begin
            TranslateMessage(M);
            DispatchMessageW(M);
          end 
        end
        else
          (*
            CN_CHAR, CN_KEYDOWN ł́Â܂܏AMsg.Result = 0
            A܂胁j[̃V[gJbgL[łAANZ
            [^[L[łȂꍇɂ WM_CHAR, WM_KEYDOWN ɉH
            čĂяB
          *)
          // CN_xxxx
          if SendMessageW(FControl.Handle, Msg, CharCode, KeyData) = 0 then
            // WM_xxxx
            SendMessageW(FControl.Handle, Msg - CN_BASE, CharCode, KeyData);
        // CancelExe, Clear \bhȂǂɂ钆f̔
        if FStatus <> msRunning then
        begin
          if FShowDialog then
            MessageDlg('Cancel Execute ' + Self.Name, mtInformation, [mbOk], 0);
          OldKeyState[VK_SHIFT] := 0;
          OldKeyState[VK_CONTROL] := 0;
          OldKeyState[VK_MENU] := 0;
          Break;
        end;
        SetKeyboardState(OldKeyState); // (*1)
      end;
  finally
    // L[{[h̏ԕA
    SetKeyboardState(OldKeyState);
    // Ԃҋ@
    Status := msWaiting;
  end;
end;

procedure TKeyMacro.LoadFromFile(const FileName: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
  try
    LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TKeyMacro.LoadFromStream(Stream: TStream);
begin
  StreamToMacroData(Stream);
end;

procedure TKeyMacro.SaveToFile(const FileName: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(FileName, fmCreate);
  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TKeyMacro.SaveToStream(Stream: TStream);
begin
  MacroDataToStream(Stream);
end;

procedure TKeyMacro.MacroDataToStream(Stream: TStream);
var
  I: Integer;
  S: String;
  List: TStringList;
begin
  List := TStringList.Create;
  try
    for I := 0 to FList.Count - 1 do
      with PKeyMacroData(FList.Items[I])^ do
      begin
        case Msg of
          WM_IME_CHAR: S := 'WM_IME_CHAR: ';
          CN_KEYDOWN:  S := 'CN_KEYDOWN: ';
          CN_CHAR:     S := 'CN_CHAR: ';
        end;
        S := S + IntToStr(Msg) + ', ' +
             'CharCode: ' + IntToStr(CharCode) + ', ' +
             'KeyData: ' + IntToStr(KeyData) + ', ' +
             'Shift: ' + IntToStr(Shift) + ', ' +
             'Ctrl: ' + IntToStr(Ctrl) + ', ' +
             'Alt: ' + IntToStr(Alt);
        if Msg <> CN_KEYDOWN then
        begin
          // f[^͍sɃRg`Ŗ
          S := S + ' // ';
          if CharCode > 255 then
            S := S + Char(HiByte(CharCode)) + Char(LoByte(CharCode))
          else
            if CharCode >= 32 then
              S := S + Char(CharCode)
            else
              S := S + '#' + IntToStr(CharCode);
        end;
        List.Add(S);
      end;
    List.SaveToStream(Stream);
  finally
    List.Free;
  end;
end;

function TKeyMacro.StreamToMacroData(Stream: TStream): Boolean;
var
  SourceLine, I: Integer;
  Parser: TParser;
  Message: TMessage;
  Buf: PKeyMacroData;
begin
  Result := True;
  try
    Clear;
    SourceLine := 0;
    I := 0;
    FillChar(Message, SizeOf(TMessage), 0);
    Buf := nil;
    Parser := TParser.Create(Stream);
    try
      // Parser.SourceLine ̏l͂P cf Classes.pas
      while Parser.Token <> toEof do
      begin
        if SourceLine <> Parser.SourceLine then
        begin
          // error trap
          if (SourceLine > 0) and (I < 6) then
            Abort;
          SourceLine := Parser.SourceLine;
          Buf := FList.Add(Message);
          I := 0;
        end;
        if Parser.Token = toInteger then
        begin
          Inc(I);
          case I of
            1: Buf.Msg := StrToIntDef(Parser.TokenString, 0);
            2: Buf.CharCode := StrToIntDef(Parser.TokenString, 0);
            3: Buf.KeyData := StrToIntDef(Parser.TokenString, 0);
            4: Buf.Shift := StrToIntDef(Parser.TokenString, 0);
            5: Buf.Ctrl := StrToIntDef(Parser.TokenString, 0);
            6: Buf.Alt := StrToIntDef(Parser.TokenString, 0);
            // Vڈȍ~ toInteger ͖
          end;
        end;
        Parser.NextToken;
      end;
    finally
      Parser.Free;
    end;
    Status := msWaiting;
  except
    Result := False;
    Clear;
    if FShowDialog then
      MessageDlg('Invalid data loaded.' + Self.Name, mtError, [mbOk], 0);
  end;
end;

end.

