//http://homepage2.nifty.com/Km/onig.htm
//http://www.geocities.jp/kosako3/oniguruma/doc/API.ja.txt
unit OnigurumaClasses;

interface

uses
  Windows, Oniguruma, StringBuffer;

type
  TOniRegexp = class
  private
    FCompiled: boolean;
    FRegExp: OnigRegex;
    FRegion: POnigRegion;
    FExpression: WideString;
    FIgCase: Cardinal;
    FInputed: boolean;
    FInput: integer;
    FMatchPos: integer;
    FMatchLen: integer;
    FMatchEnd: integer;
    FSB: TStringBuffer;
    procedure SetInputString(const Value: PWideChar);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Compile(const AExpression: WideString; AIgCase: boolean);
    property Compiled: boolean read FCompiled;
    property InputString : PWideChar write SetInputString;
    function Exec (AOffset: integer; ALastPoint, ALimit: integer; LToR: boolean; Replace1st: boolean) : boolean;
    property MatchPos: integer read FMatchPos;
    property MatchEnd: integer read FMatchEnd;
    property MatchLen: integer read FMatchLen;
    function Substitute (const Value : WideString) : WideString;
    class function PreSubstitute(const Value: WideString): WideString;
    class function Copyright:WideString;
  end;

implementation

var
  FLoaded: boolean;
  FHandle: THandle;


function OnigLoadLibrary: Boolean;
begin
  FHandle := LoadLibraryW('onig.dll');
  Result := (FHandle <> 0);
  if Result then
  begin
    @ONIG_ENCODING_UTF16_LE               := GetProcAddress(FHandle, 'OnigEncodingUTF16_LE');
    @ONIG_SYNTAX_DEFAULT                  := GetProcAddress(FHandle, 'OnigDefaultSyntax');
    @onig_new                             := GetProcAddress(FHandle, 'onig_new');
    @onig_free                            := GetProcAddress(FHandle, 'onig_free');
    @onig_search                          := GetProcAddress(FHandle, 'onig_search');
    @onig_region_new                      := GetProcAddress(FHandle, 'onig_region_new');
    @onig_region_free                     := GetProcAddress(FHandle, 'onig_region_free');
    @onig_version                         := GetProcAddress(FHandle, 'onig_version');
    @onig_copyright                       := GetProcAddress(FHandle, 'onig_copyright');
  end;
end;

procedure OnigFreeLibrary;
begin
  if (FHandle <> 0) then
    FreeLibrary(FHandle);
end;

{ TOniRegexp }

procedure TOniRegexp.Compile(const AExpression: WideString; AIgCase: boolean);
var
  len: integer;
  AInfo: OnigErrorInfo;
begin
  if not FLoaded then exit;
  FExpression := AExpression;
  if AIgCase then FIgCase := ONIG_OPTION_IGNORECASE else FIgCase := 0;
  len := Length(FExpression);
  if FCompiled then onig_free(FRegExp);
  FCompiled := onig_new(@FRegExp,
                        POnigUChar(FExpression),
                        POnigUChar(PWideChar(FExpression) + len),
                        ONIG_OPTION_NEGATE_SINGLELINE or
                        ONIG_OPTION_NONE or
                        //ONIG_OPTION_FIND_NOT_EMPTY or
                        FIgCase,
                        @ONIG_ENCODING_UTF16_LE,
                        POnigSyntaxType((@ONIG_SYNTAX_DEFAULT)^),
                        @AInfo) = ONIG_NORMAL;

end;

class function TOniRegexp.Copyright: WideString;
begin
  if not FLoaded then
    Result := ''
  else
    Result := onig_copyright;
end;

constructor TOniRegexp.Create;
begin
  FCompiled := false;
  FInputed := false;
  FSB := TStringBuffer.Create(0);
  if not FLoaded then FLoaded := OnigLoadLibrary;
end;

destructor TOniRegexp.Destroy;
begin
  if FInputed then onig_region_free(FRegion, 1);
  if FCompiled then onig_free(FRegExp);
  FSB.Free;
  inherited;
end;

function TOniRegexp.Exec(AOffset, ALastPoint, ALimit: integer;
  LToR: boolean; Replace1st: boolean): boolean;
var
  r: integer;
begin
  Result := false;
  if (not FCompiled) or (not FInputed) then exit;
  if LtoR then begin
    if AOffset - 1 >= ALimit then exit;
  end else ALimit := 0;
  r := onig_search(FRegExp,
                         Pointer(FInput),
                         Pointer(FInput + ALastPoint * 2),
                         Pointer(FInput + (AOffset - 1) * 2),
                         Pointer(FInput + Alimit * 2),
                         FRegion,
                         ONIG_OPTION_NONE);
  Result :=  r >= 0;
  FMatchPos := (FRegion.match_beg^ div 2) + 1;
  FMatchEnd := (FRegion.match_end^ div 2) + 1;
  FMatchLen := FMatchEnd - FMatchPos;
  if Result then begin
      if LToR then begin
        if (FMatchLen = 0) and (AOffset = FMatchPos) then begin
          if Replace1st then exit;
          Result := onig_search(FRegExp,
                         Pointer(FInput),
                         Pointer(FInput + ALastPoint * 2),
                         Pointer(FInput + (AOffset) * 2),
                         Pointer(FInput + (Alimit + 1) * 2),
                         FRegion,
                         ONIG_OPTION_NONE) >= 0;
          FMatchPos := (FRegion.match_beg^ div 2) + 1;
          FMatchEnd := (FRegion.match_end^ div 2) + 1;
          FMatchLen := FMatchEnd - FMatchPos;
        end;
      end;
  end;
end;

class function TOniRegexp.PreSubstitute(const Value: WideString): WideString;
var
  i, j, len: integer;
begin
  i := 1;
  j := 1;
  len := Length(Value);
  SetLength(Result, len);
  while i <= len do begin
    if Value[i] = #$005C then begin
      inc(i);
      if i > len then break;
      case Value[i] of
        '\': begin //GXP[v͂ŏ
          //Result[j] := '\';
          //inc(j);
          Result[j] := '\';
          inc(j);
          inc(i);
        end;
        '1'..'9': begin
          Result[j] := '$';
          inc(j);
          Result[j] := Value[i];
          inc(j);
          inc(i);
        end;
        't': begin
          Result[j] := #9;
          inc(j);
          inc(i);
        end;
        'n': begin
          Result[j] := #10;
          inc(j);
          inc(i);
        end;
        'r': begin
          Result[j] := #13;
          inc(j);
          inc(i);
        end;
        'f': begin
          Result[j] := #12;
          inc(j);
          inc(i);
        end;
        'a': begin
          Result[j] := #7;
          inc(j);
          inc(i);
        end;
        'e': begin
          Result[j] := #11;
          inc(j);
          inc(i);
        end;
        else begin
          //Result[j] := '\';
          //inc(j);
          Result[j] := Value[i];
          inc(j);
          inc(i);
        end;
      end;
    end else begin
      Result[j] := Value[i];
      inc(j);
      inc(i);
    end;
  end;//while
  SetLength(Result, j - 1);
end;

procedure TOniRegexp.SetInputString(const Value: PWideChar);
begin
  if not FLoaded then exit;
  if FInputed then onig_region_free(FRegion, 1);
  FRegion := onig_region_new;
  FInputed := true;
  FInput := integer(Value);
end;

function TOniRegexp.Substitute(const Value: WideString): WideString;
var
  i, len1: integer;
  num, AStart: integer;
begin
  i := 1;
  len1 := Length(Value);
  SetLength(Result, len1);
  FSB.Clear;
  while i <= len1 do begin
    if (Value[i] = '$') and (i + 1 <= len1) and
       (Value[i + 1] >= '0') and (Value[i + 1] <= '9')then begin
      inc(i);
      num := Word(Value[i]) - $30;
      inc(i);
      if num < FRegion.num_regs then begin
        AStart := (PIntegerArray(FRegion.match_beg))[num] div 2;
        FSB.Append(PWideChar(FInput) + AStart, (PIntegerArray(FRegion.match_end))[num] div 2 - AStart);
      end;
    end else begin
      FSB.Append(Value[i]);
      inc(i);
    end;
  end;//while
  Result := FSB.ToString;
end;

initialization
finalization
  if FLoaded then OnigFreeLibrary;
end.
