unit CBWatch;

interface

uses
  Windows, Messages, Classes, Forms;

type
  TClipboardWatcher = class(TComponent)
  private
    FWindowHandle: HWND;
    FNextWindowHandle: HWND;
    FOnClipChange: TNotifyEvent;  // OnClipChange CxgtB[h
    FEnabled: Boolean;
    FRegistered: Boolean;
    FSkip: Boolean;  // ŏ OnClipChange CxgXLbṽׂtbO
    procedure UpdateClip;
    procedure SetEnabled(const Value: Boolean);
    procedure SetOnClipChange(const Value: TNotifyEvent);
    procedure WndProc(var Message: TMessage);
  protected
    procedure ClipChange; dynamic;  // OnClipChange Cxg\bh
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Enabled: Boolean read FEnabled write SetEnabled default False;
    property OnClipChange: TNotifyEvent read FOnClipChange write SetOnClipChange;
  end;

procedure Register;

implementation

{ Register }

procedure Register;
begin
  RegisterComponents('ToolBox', [TClipboardWatcher]);
end;

{ TClipboardWatcher }

constructor TClipboardWatcher.Create(AOwner: TComponent); //override;
begin
  inherited Create(AOwner);
  FWindowHandle := Classes.AllocateHWnd(WndProc);
  FEnabled := False;
  FSkip := False;  // ŏ OnClipChange CxgXLbṽׂtbO
end;

destructor TClipboardWatcher.Destroy; //override;
begin
  SetEnabled(False);
  Classes.DeallocateHWnd(FWindowHandle);
  inherited Destroy;
end;

procedure TClipboardWatcher.UpdateClip;
begin
  if FRegistered = FEnabled and Assigned(FOnClipChange) then
    Exit;
  FRegistered := not FRegistered;
  if FRegistered then
    FNextWindowHandle := SetClipboardViewer(FWindowHandle)
  else
    ChangeClipboardChain(FWindowHandle, FNextWindowHandle);
end;

{ Enabled vpeB False  True ɂƁANbv{[h }
{ ωȂƂK OnClipChange Cxg񔭐̂ŁA }
{ ŏ̃CxgXLbv܂B}
procedure TClipboardWatcher.SetEnabled(const Value: Boolean);
begin
  if (FEnabled = False) and (Value = True) then
    FSkip := True
  else
    FSkip := False;

  FEnabled := Value;
  UpdateClip;
end;

procedure TClipboardWatcher.SetOnClipChange(const Value: TNotifyEvent);
begin
  FOnClipChange := Value;
  UpdateClip;
end;

procedure TClipboardWatcher.WndProc(var Message: TMessage);
begin
  with Message do
    case Msg of
      WM_CHANGECBCHAIN:
        try
          with TWMChangeCBChain(Message) do
            if Remove = FNextWindowHandle then
              FNextWindowHandle := Next
            else if FNextWindowHandle <> 0 then
              SendMessage(FNextWindowHandle, Msg, wParam, lParam);
        except
          Application.HandleException(Self);
        end;
      WM_DRAWCLIPBOARD:
        try
          if FSkip = False then
            ClipChange  // OnClipChange CxgŔ
          else  // ŏɔ OnClipChange CxgXLbvB
            FSkip := False;
          SendMessage(FNextWindowHandle, Msg, wParam, lParam);
        except
          Application.HandleException(Self);
        end;
    else
      Result := DefWindowProc(FWindowHandle, Msg, wParam, lParam);
    end;
end;

procedure TClipboardWatcher.ClipChange;  // dynamic;
begin
  if Assigned(FOnClipChange) then
    FOnClipChange(Self);
end;

end.

