unit wsFileOp;

interface

uses Windows;

const
{ File attribute constants }

  faReadOnly  = $00000001;
  faHidden    = $00000002;
  faSysFile   = $00000004;
  faVolumeID  = $00000008;
  faDirectory = $00000010;
  faArchive   = $00000020;
  faAnyFile   = $0000003F;
  PathDelim  = '\';
  DriveDelim = ':';

type
  TSearchRecW = record
    Time: Integer;
    Size: Integer;
    Attr: Integer;
    Name: widestring;
    ExcludeAttr: Integer;
    FindHandle: THandle;
    FindData: TWin32FindDataW;
  end;

function FileExistsW(const FileName: widestring): Boolean;
function DirectoryExistsW(const Directory: widestring): Boolean;
function ExpandFileNameW(const FileName: widestring): widestring;
function ExtractFileNameW(const FileName: widestring): widestring;
function ExtractFileDirW(const FileName: widestring): widestring;
function FileExists2(const F:widestring):boolean;
function DirectoryExists2(const F:widestring):boolean;
function GetLongPathOfW(const AFileName: widestring): widestring;
function ChangeFileExtW(const FileName: WideString; const Extension: Widestring): widestring;
function ParamStrW(Index: Integer): widestring;
function ParamCountW: Integer;
function FindFirstW(const Path: widestring; Attr: Integer;
  var F: TSearchRecW): Integer;
function FindNextW(var F: TSearchRecW): Integer;
procedure FindCloseW(var F: TSearchRecW);
function FileIsReadOnlyW(const FileName: widestring): Boolean;
function ExtractFilePathW(const FileName: widestring): widestring;
function ExtractShortPathNameW(const FileName: widestring): widestring;
function ExtractFileExtW(const FileName: widestring): widestring;
function GetCurrentDirW: widestring;
function SetCurrentDirW(const Dir: widestring): Boolean;


implementation

function ExtractShortPathNameW(const FileName: widestring): widestring;
begin
  SetLength(Result,MAX_PATH);
  SetLength(Result,GetShortPathNameW(pwchar(FileName),pwchar(Result),Length(Result)));
end;

function FileIsReadOnlyW(const FileName: widestring): Boolean;
begin
  Result := (GetFileAttributesW(PWChar(FileName)) and faReadOnly) <> 0;
end;


function GetParamStrW(P: PWChar; var Param: widestring): PWChar;
var
  i, Len: Integer;
  Start, S, Q: PWChar;
begin
  while True do
  begin
    while (P[0] <> #0) and (P[0] <= ' ') do
      P := CharNextW(P);
    if (P[0] = '"') and (P[1] = '"') then Inc(P, 2) else Break;
  end;
  Len := 0;
  Start := P;
  while P[0] > ' ' do
  begin
    if P[0] = '"' then
    begin
      P := CharNextW(P);
      while (P[0] <> #0) and (P[0] <> '"') do
      begin
        Q := CharNextW(P);
        Inc(Len, Q - P);
        P := Q;
      end;
      if P[0] <> #0 then
        P := CharNextW(P);
    end
    else
    begin
      Q := CharNextW(P);
      Inc(Len, Q - P);
      P := Q;
    end;
  end;

  SetLength(Param, Len);

  P := Start;
  S := Pointer(Param);
  i := 0;
  while P[0] > ' ' do
  begin
    if P[0] = '"' then
    begin
      P := CharNextW(P);
      while (P[0] <> #0) and (P[0] <> '"') do
      begin
        Q := CharNextW(P);
        while P < Q do
        begin
//          S[i] := P^;
          S[i] := P[0];
          Inc(P);
          Inc(i);
        end;
      end;
      if P[0] <> #0 then P := CharNextW(P);
    end
    else
    begin
      Q := CharNextW(P);
      while P < Q do
      begin
//        S[i] := P^;
        S[i] := P[0];
        Inc(P);
        Inc(i);
      end;
    end;
  end;

  Result := P;
end;

function ParamStrW(Index: Integer): widestring;
var
  P: PWChar;
begin
  Result := '';
  if Index = 0 then begin
    SetLength(Result,Max_Path);
    SetLength(Result, GetModuleFileNameW(0, PWChar(Result), Length(Result)));
  end else
  begin
    P := GetCommandLineW;
    while True do
    begin
      P := GetParamStrW(P, Result);
      if (Index = 0) or (Result = '') then Break;
      Dec(Index);
    end;
  end;
end;

function ParamCountW: Integer;
var
  P: PWChar;
  S: widestring;
begin
  Result := 0;
  P := GetParamStrW(GetCommandLineW, S);
  while True do
  begin
    P := GetParamStrW(P, S);
    if S = '' then Break;
    Inc(Result);
  end;
end;

function ChangeFileExtW(const FileName, Extension: widestring): widestring;
var
  I,j: Integer;
begin
  i:=0;
  for j:=1 to Length(Filename) do
    if (FileName[j]=PathDelim)or(FileName[j]=DriveDelim)
      or(FileName[j]='.') then i:=j;
  if (I = 0) or (FileName[I] <> '.') then I := MaxInt;
  Result := Copy(FileName, 1, I - 1) + Extension;
end;

function ExtractFileExtW(const FileName: widestring): widestring;
var
  I,j: Integer;
begin
  i:=0;
  for j:=1 to Length(Filename) do
    if (FileName[j]=PathDelim)or(FileName[j]=DriveDelim)
      or(FileName[j]='.') then i:=j;
  if (I > 0) and (FileName[I] = '.') then
    Result := Copy(FileName, I, MaxInt) else
    Result := '';
end;


type
  LongRec = packed record
    case Integer of
      0: (Lo, Hi: Word);
      1: (Words: array [0..1] of Word);
      2: (Bytes: array [0..3] of Byte);
  end;


function FindMatchingFileW(var F: TSearchRecW): Integer;
var
  LocalFileTime: TFileTime;
begin
  with F do
  begin
    while FindData.dwFileAttributes and ExcludeAttr <> 0 do
      if not FindNextFileW(FindHandle, FindData) then
      begin
        Result := GetLastError;
        Exit;
      end;
    FileTimeToLocalFileTime(FindData.ftLastWriteTime, LocalFileTime);
    FileTimeToDosDateTime(LocalFileTime, LongRec(Time).Hi,
      LongRec(Time).Lo);
    Size := FindData.nFileSizeLow;
    Attr := FindData.dwFileAttributes;
    Name := FindData.cFileName;
  end;
  Result := 0;
end;

procedure FindCloseW(var F: TSearchRecW);
begin
  if F.FindHandle <> INVALID_HANDLE_VALUE then
  begin
    Windows.FindClose(F.FindHandle);
    F.FindHandle := INVALID_HANDLE_VALUE;
  end;
end;

function FindFirstW(const Path: widestring; Attr: Integer;
  var F: TSearchRecW): Integer;
const
  faSpecial = faHidden or faSysFile or faVolumeID or faDirectory;
begin
  F.ExcludeAttr := not Attr and faSpecial;
  F.FindHandle := FindFirstFileW(PWChar(Path), F.FindData);
  if F.FindHandle <> INVALID_HANDLE_VALUE then
  begin
    Result := FindMatchingFileW(F);
    if Result <> 0 then FindCloseW(F);
  end else
    Result := GetLastError;
end;

function FindNextW(var F: TSearchRecW): Integer;
begin
  if FindNextFileW(F.FindHandle, F.FindData) then
    Result := FindMatchingFileW(F) else
    Result := GetLastError;
end;

function IsPathDelimiter(const S: widestring; Index: Integer): Boolean;
begin
  Result := (Index > 0) and (Index <= Length(S)) and (S[Index] = PathDelim);
end;

function IncludeTrailingPathDelimiter(const S: widestring): widestring;
begin
  Result := S;
  if not IsPathDelimiter(Result, Length(Result)) then
    Result := Result + PathDelim;
end;

function ExtractFilePathW(const FileName: widestring): widestring;
var
  I,j: Integer;
begin
  i:=0;
  for j:=1 to Length(Filename) do
    if (FileName[j]=PathDelim)or(FileName[j]=DriveDelim) then i:=j;
  Result := Copy(FileName, 1, I);
end;

function ExtractFileDirW(const FileName: widestring): widestring;
var
  I,j: Integer;
begin
  i:=0;
  for j:=1 to Length(Filename) do
    if (FileName[j]=PathDelim)or(FileName[j]=DriveDelim) then i:=j;
  if (I > 1) and (FileName[I] = PathDelim) and
    (not ((FileName[i-1]=PathDelim)or(FileName[i-1]=DriveDelim))) then Dec(I);
  Result := Copy(FileName, 1, I);
end;

function GetLongPathOfW(const AFileName: widestring): widestring;
  function GetLongNameOf(const AFilePath: widestring): widestring;
  var
    SearchRec: TSearchRecW;
  begin
    if Length(AFilePath) = 3 then//hCûw肳ꂽ
    begin
      Result := CharUpperW(PWChar(AFilePath));
    end
    else begin
      if FindFirstW(AFilePath, faAnyFile, SearchRec) = 0 then
      begin
        //AFilePathƂ
        Result := IncludeTrailingPathDelimiter(GetLongNameOf(
          ExtractFileDirW(AFilePath))) + SearchRec.Name;
      end
      else begin
        if (Length(AFilePath) >= 2) and (AFilePath[1] = '\') and (AFilePath[2] = '\') then
        Result :=  AFilePath else//\\w肳ꂽ
        Result := '';
      end;
      FindCloseW(SearchRec);
    end;
  end;

begin
  Result := '';
  if FileExistsW(AFileName) then
  begin
    Result := GetLongNameOf(ExpandFileNameW(AFileName));
  end;
end;

function InternalGetDiskSpace(Drive: Byte;
  var TotalSpace, FreeSpaceAvailable: Int64): Bool;
var
  RootPath: array[0..4] of WChar;
  RootPtr: PWChar;
begin
  RootPtr := nil;
  if Drive > 0 then
  begin
    RootPath[0] := WChar(Drive + $40);
    RootPath[1] := ':';
    RootPath[2] := '\';
    RootPath[3] := #0;
    RootPtr := RootPath;
  end;
  Result := GetDiskFreeSpaceExW(RootPtr, FreeSpaceAvailable, TotalSpace, nil);
end;

function DiskSize(Drive: Byte): Int64;
var
  FreeSpace: Int64;
begin
  if not InternalGetDiskSpace(Drive, Result, FreeSpace) then
    Result := -1;
end;

Function IsDriveReady( drive:WChar ) : Boolean;
Var
  oldmode   : word;
  searchrec : tSearchRecW;
Begin
  if drive='\' then begin
    Result:=true;
    exit;
  end;
  drive   := CharUpperW(PWChar(WideString( drive )))^;
  oldmode := SetErrorMode( SEM_FAILCRITICALERRORS );  (* G[ *)
  try //ׂ?
    result  := DiskSize( ord( drive )-$40 )<>-1;
    if result and ( GetDriveTypeW( pWchar( widestring(drive+':\' )) ) in [DRIVE_REMOTE,DRIVE_CDROM] ) then begin
      try
        result := FindFirstW( drive+':\*.*',$3f,searchrec )=0;
      finally
        FindCloseW( searchrec );
      end;
    end;
  finally
    SetErrorMode( oldmode );
  end;                            (* ʏ폈 *)
End;

function FileExists2(const F:widestring):boolean;
begin
  if Length(f)=0 then Result:=false else begin
    if (ExpandFileNameW(f)[1] = '\') or
      IsDriveReady(ExpandFileNameW(f)[1]) then Result:=FileExistsW(f) else Result:=false;
  end;//
end;

function DirectoryExists2(const F:widestring):boolean;
begin
  if Length(f)=0 then Result:=false else begin
    if (ExpandFileNameW(f)[1] = '\') or
      IsDriveReady(ExpandFileNameW(f)[1]) then Result:=DirectoryExistsW(f) else Result:=false;
  end;//
end;

function ExtractFileNameW(const FileName: widestring): widestring;
var
  I,j: Integer;
begin
  i:=0;
  for j:=1 to Length(Filename) do
    if (FileName[j]=PathDelim)or(FileName[j]=DriveDelim) then i:=j;
  Result := Copy(FileName, I + 1, MaxInt);
end;


function ExpandFileNameW(const FileName: widestring): widestring;
var
  FName: PwChar;
begin
  SetLength(Result,MAX_PATH);
  SetLength(Result,GetFullPathNameW(pwchar(FileName),MAX_PATH,pwchar(Result),FName));
end;

function FileExistsW(const FileName: widestring): Boolean;
var
  Code: Integer;
begin
  Code := GetFileAttributesW(PWChar(FileName));
  Result := (Code <> -1) and (FILE_ATTRIBUTE_DIRECTORY and Code = 0);
end;

function DirectoryExistsW(const Directory: widestring): Boolean;
var
  Code: Integer;
begin
  Code := GetFileAttributesW(PWChar(Directory));
  Result := (Code <> -1) and (FILE_ATTRIBUTE_DIRECTORY and Code <> 0);
end;

function SetCurrentDirW(const Dir: widestring): Boolean;
begin
  Result := SetCurrentDirectoryW(PWChar(Dir));
end;

function GetCurrentDirW: widestring;
begin
  SetLength(Result,MAX_PATH);
  SetLength(Result,GetCurrentDirectoryW(MAX_PATH,pWchar(Result)));
end;

end.
